@@ -77,6 +77,7 @@
 #endif
 
 #define DEBUG_SYSDEP		0x1
+#define DEBUG_SERVER		0x2
 #define DEBUG_MAIL			0x10
 #define DEBUG_NET			0x20
 #define DEBUG_TIMER			0x40
@@ -115,6 +116,7 @@ struct GkrellmdConfig
 				inet_interval;
 
 	gboolean	without_libsensors;
+	gboolean	use_acpi_battery;
 
 	gboolean	list_plugins,
 				log_plugins;
