@@ -21,14 +21,13 @@
 
 #include "gkrellmd.h"
 #include "gkrellmd-private.h"
+#include "log-private.h"
 
-#ifdef WIN32
-	#include "win32-gui.h"
-#endif
+#if !defined(WIN32)
+	#include <syslog.h>
+#endif // !WIN32
 
-//#include "../src/inet.h"
-
-// we do have addrinfo on win32 but do not have getaddrinfo(), doh
+// win32 defines addrinfo but only supports getaddrinfo call on winxp or newer
 #if !defined(HAVE_GETADDRINFO) && !defined(WIN32)
 struct addrinfo
 	{
@@ -41,7 +40,7 @@ struct addrinfo
 	struct	sockaddr *ai_addr;	/* binary address */
 	struct	addrinfo *ai_next;	/* next structure in linked list */
 	};
-#endif // HAVE_GETADDRINFO && !WIN32
+#endif // !HAVE_GETADDRINFO
 
 #if !defined(IPV6_V6ONLY) && defined(IPV6_BINDV6ONLY)
 #define	IPV6_V6ONLY	IPV6_BINDV6ONLY
@@ -55,16 +54,131 @@ GList			*gkrellmd_client_list,
 
 static GList	*allow_host_list;
 
+#if !defined(WIN32)
 static gboolean	detach_flag;
 
-#if !defined(WIN32)
 struct
 	{
 	uid_t	uid;
 	uid_t	gid;
 	}
 	drop_privs = { 0, 0 };
+#endif /* !defined(WIN32) */
+
+
+#if defined(WIN32)
+/*
+	Flag that determines if gkrellmd was started as a console app (FALSE)
+	or as a service (TRUE)
+*/
+static gboolean service_is_one = FALSE;
+
+// Flag that is TRUE while gkrellmd should stay in its main loop
+static gboolean service_running = FALSE;
+
+// Unique name for the installed windows service (do not translate!)
+static wchar_t* service_name = L"gkrellmd";
+
+// User visible name for the installed windows service
+static wchar_t* service_display_name = L"GKrellM Daemon";
+
+/*
+	Current service status if running as a service, may be stopped or running
+	(pausing is not supported)
+*/
+static SERVICE_STATUS service_status;
+
+/*
+	Handle that allows changing the service status.
+	Main use is to stop the running service.
+*/
+static SERVICE_STATUS_HANDLE service_status_handle = 0;
+
+/*
+	Handle to our event log source.
+	The Windows Event Log is used as a replacement for syslog-logging.
+*/
+static HANDLE h_event_log = NULL;
+
+#endif /* defined(WIN32) */
+
+
+static gboolean
+gkrellmd_syslog_init()
+	{
+#if defined(WIN32)
+	h_event_log = RegisterEventSourceW(NULL, service_name);
+	if (h_event_log == NULL)
+		{
+		g_warning("Cannot register event source for logging into Windows Event Log.\n");
+		return FALSE;
+		}
+#else
+	// Unix needs no logging initialization
 #endif
+	return TRUE;
+	}
+
+static gboolean
+gkrellmd_syslog_cleanup()
+	{
+#if defined(WIN32)
+	if (h_event_log)
+		DeregisterEventSource(h_event_log);
+	h_event_log = NULL;
+#else
+	// Unix needs no further logging cleanup
+#endif
+	return TRUE;
+	}
+
+static void gkrellmd_syslog_log(GLogLevelFlags log_level, const gchar *message)
+	{
+#if defined(WIN32)
+	WORD event_type;
+	const char *p_buf[1];
+
+	// Abort if event source is missing
+	if (h_event_log == NULL)
+		return;
+
+	event_type = EVENTLOG_INFORMATION_TYPE;
+	if (log_level & G_LOG_LEVEL_WARNING)
+		event_type = EVENTLOG_WARNING_TYPE;
+	if (log_level & G_LOG_LEVEL_CRITICAL || log_level & G_LOG_LEVEL_ERROR)
+		event_type = EVENTLOG_ERROR_TYPE;
+
+	p_buf[0] = message;
+
+	ReportEventA(
+		h_event_log,  // Event source handle (HANDLE)
+		event_type,   // Event type (WORD)
+		0,            // Event category (WORD)
+		0,            // Event identifier (DWORD)
+		NULL,         // user security identifier (PSID)
+		1,            // Number of substitution strings (WORD)
+		0,            // Data Size (DWORD)
+		p_buf,        // Pointer to strings
+		NULL          // Pointer to Data
+		);
+#else
+	int facility_priority;
+
+	// default to info and override with other states if they are more important
+	facility_priority = LOG_MAKEPRI(LOG_DAEMON, LOG_INFO);
+	if (log_level & G_LOG_LEVEL_DEBUG)
+		facility_priority = LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG);
+	if (log_level & G_LOG_LEVEL_WARNING)
+		facility_priority = LOG_MAKEPRI(LOG_DAEMON, LOG_WARNING);
+	if (log_level & G_LOG_LEVEL_ERROR)
+		facility_priority = LOG_MAKEPRI(LOG_DAEMON, LOG_ERR);
+	if (log_level & G_LOG_LEVEL_CRITICAL)
+		facility_priority = LOG_MAKEPRI(LOG_DAEMON, LOG_CRIT);
+
+	syslog(facility_priority, message);
+#endif // defined(WIN32)
+	} // gkrellmd_syslog_log()
+
 
 static void
 make_pidfile(void)
@@ -81,7 +195,7 @@ make_pidfile(void)
 		fclose(f);
 		}
 	else
-		fprintf(stderr, "gkrellmd: Can't create pidfile %s\n", _GK.pidfile);
+		g_warning("Can't create pidfile %s\n", _GK.pidfile);
 #endif
 	}
 
@@ -94,11 +208,22 @@ remove_pidfile(void)
 #endif
 	}
 
+static void
+gkrellmd_cleanup()
+	{
+	gkrellm_sys_main_cleanup();
+	gkrellm_log_cleanup();
+	remove_pidfile();
+	}
+
 static void
 cb_sigterm(gint sig)
 	{
-	remove_pidfile();
-    exit(0);
+	g_message("GKrellM Daemon %d.%d.%d%s: Exiting normally\n",
+			GKRELLMD_VERSION_MAJOR, GKRELLMD_VERSION_MINOR,
+			GKRELLMD_VERSION_REV, GKRELLMD_EXTRAVERSION);
+	gkrellmd_cleanup();
+	exit(0);
 	}
 
 gint
@@ -116,7 +241,7 @@ gkrellmd_send_to_client(GkrellmdClient *client, gchar *buf)
 	if (n < 0 && errno == EPIPE)
 		{
 		if (_GK.verbose)
-			printf("Write on closed pipe to host %s\n", client->hostname);
+			g_print("Write on closed pipe to host %s\n", client->hostname);
 		client->alive = FALSE;
 		}
 	return n;
@@ -360,7 +485,7 @@ allow_host(GkrellmdClient *client, struct sockaddr *sa, socklen_t salen)
 
 	snprintf(buf, sizeof(buf), _("Connection not allowed from %s\n"),
 			hostname ? hostname : addr);
-	fprintf(stderr, "gkrellmd: %s", buf),
+	g_warning(buf),
 	gkrellmd_send_to_client(client, "<error>\n");
 	gkrellmd_send_to_client(client, buf);
 	return FALSE;
@@ -385,8 +510,7 @@ accept_client(gint fd, struct sockaddr *sa, socklen_t salen)
 		{
 		if (client_limit)
 			{
-			fprintf(stderr, _("gkrellmd: too many clients, rejecting %s\n"),
-					client->hostname);
+			g_message(_("Too many clients, rejecting %s\n"), client->hostname);
 			gkrellmd_send_to_client(client,
 						"<error>\nClient limit exceeded.\n");
 			}
@@ -402,7 +526,7 @@ accept_client(gint fd, struct sockaddr *sa, socklen_t salen)
 	//getline(fd, buf, sizeof(buf));
 
 	if (_GK.verbose)
-		printf(_("connect string from client: %s\n"), buf);
+		g_print(_("connect string from client: %s\n"), buf);
 
 	if (   sscanf(buf, "%31s %d.%d.%d", name, &client->major_version,
 					&client->minor_version, &client->rev_version) == 4
@@ -412,8 +536,7 @@ accept_client(gint fd, struct sockaddr *sa, socklen_t salen)
 		gkrellmd_client_list = g_list_append(gkrellmd_client_list, client);
 		return client;
 		}
-	fprintf(stderr, _("gkrellmd: bad connect line from %s: %s\n"),
-				client->hostname, buf);
+	g_warning(_("Bad connect line from %s: %s\n"), client->hostname, buf);
 	gkrellmd_send_to_client(client, "<error>\nBad connect string!");
 
 	g_free(client->hostname);
@@ -432,8 +555,7 @@ remove_client(gint fd)
 		client = (GkrellmdClient *) list->data;
 		if (client->fd == fd)
 			{
-			if (_GK.verbose)
-				printf("Removing client %s\n", client->hostname);
+			g_message(_("Removing client %s\n"), client->hostname);
 			close(fd);
 			g_free(client->hostname);
 			g_free(client);
@@ -451,13 +573,24 @@ parse_config(gchar *config, gchar *arg)
 		gkrellm_free_glist_and_data(&allow_host_list);
 		return 0;
 		}
+	if (!strcmp(config, "syslog"))
+		{
+		gkrellm_log_register(gkrellmd_syslog_log, gkrellmd_syslog_init,
+			gkrellmd_syslog_cleanup);
+		return 0;
+		}
+#if !defined(WIN32)
 	if (!strcmp(config, "detach") || !strcmp(config, "d"))
 		{
 		detach_flag = TRUE;
 		return 0;
 		}
+#endif
+
+	// All following options take one argument that should be passed in arg
 	if (!arg || !*arg)
 		return -1;
+
 	if (!strcmp(config, "update-hz") || !strcmp(config, "u"))
 		_GK.update_HZ = atoi(arg);
 	else if (!strcmp(config, "port") || !strcmp(config, "P"))
@@ -489,6 +622,8 @@ parse_config(gchar *config, gchar *arg)
 		_GK.net_timer = g_strdup(arg);
 	else if (!strcmp(config, "debug-level") || !strcmp(config, "debug"))
 		_GK.debug_level = (gint) strtoul(arg, NULL, 0);
+	else if (!strcmp(config, "logfile"))
+		gkrellm_log_set_filename(arg);
 #if !defined(WIN32)
 	else if (!strcmp(config, "pidfile"))
 		_GK.pidfile = g_strdup(arg);
@@ -523,9 +658,11 @@ load_config(gchar *path)
 	{
 	FILE			*f;
 	PluginConfigRec	*cfg;
-	gchar			buf[128], config[32], arg[128];
+	gchar			buf[128+32+2], config[32], arg[128];
 	gchar			*s, *plugin_config_block = NULL;
 
+	//g_print("Trying to load config from file '%s'\n", path);
+
 	f = fopen(path, "r");
 	if (!f)
 		return;
@@ -598,8 +735,12 @@ static void
 read_config(void)
 	{
 	gchar	*path;
+#if defined(WIN32)
+	gchar *install_path;
+#endif
 
 	_GK.update_HZ = 3;
+	_GK.debug_level = 0;
 	_GK.max_clients = 2;
 	_GK.server_port = GKRELLMD_SERVER_PORT;
 
@@ -608,25 +749,37 @@ read_config(void)
 	_GK.inet_interval = 1;
 
 #if defined(GKRELLMD_SYS_ETC)
-	path = g_strdup_printf("%s/%s", GKRELLMD_SYS_ETC, GKRELLMD_CONFIG);
+	path = g_build_filename(GKRELLMD_SYS_ETC, GKRELLMD_CONFIG, NULL);
 	load_config(path);
 	g_free(path);
 #endif
 
 #if defined(GKRELLMD_LOCAL_ETC)
-	path = g_strdup_printf("%s/%s", GKRELLMD_LOCAL_ETC, GKRELLMD_CONFIG);
+	path = g_build_filename(GKRELLMD_LOCAL_ETC, GKRELLMD_CONFIG, NULL);
 	load_config(path);
 	g_free(path);
 #endif
 
-	_GK.homedir = (gchar *) g_get_home_dir();
-	if (!_GK.homedir)
-		_GK.homedir = ".";
-#if !defined(WIN32)
-	path = g_strdup_printf("%s/.%s", _GK.homedir, GKRELLMD_CONFIG);
+// on windows also load config from INSTALLDIR/etc/gkrellmd.conf
+#if defined(WIN32)
+#if GLIB_CHECK_VERSION(2,16,0)
+	install_path = g_win32_get_package_installation_directory_of_module(NULL);
+	path = g_build_filename(install_path, "etc", GKRELLMD_CONFIG, NULL);
 #else
-	path = g_strdup_printf("%s/%s", _GK.homedir, GKRELLMD_CONFIG);
+	// deprecated since glib 2.16
+	install_path = g_win32_get_package_installation_subdirectory(NULL, NULL, "etc");
+	path = g_build_filename(install_path, GKRELLMD_CONFIG, NULL);
 #endif
+	load_config(path);
+	g_free(install_path);
+	g_free(path);
+#endif
+
+	_GK.homedir = (gchar *) g_get_home_dir();
+	if (_GK.homedir == NULL)
+		_GK.homedir = ".";  // FIXME: doesn't look right to me
+
+	path = g_build_filename(_GK.homedir, GKRELLMD_USER_CONFIG, NULL);
 	load_config(path);
 	g_free(path);
 	}
@@ -634,94 +787,109 @@ read_config(void)
 static void
 usage(void)
 	{
-	printf(_("usage: gkrellmd [options]\n"));
-	printf(_("options:\n"));
-	printf(_("   -u, --update-hz F         Monitor update frequency\n"));
-	printf(_("   -m, --max-clients N       Number of simultaneous clients\n"));
-	printf(_("   -A, --address A           Address of network interface to listen on\n"));
-	printf(_("   -P, --port P              Server port to listen on\n"));
-	printf(_("   -a, --allow-host host     Allow connections from specified hosts\n"));
-	printf(_("   -c, --clear-hosts         Clears the current list of allowed hosts\n"));
-	printf(_("       --io-timeout N        Close connection after N seconds of no I/O\n"));
-	printf(_("       --reconnect-timeout N Try to connect every N seconds after\n"
+#if defined(WIN32)
+
+	g_print(_("usage: gkrellmd command [options]\n"));
+	g_print(_("commands:\n"));
+	g_print(_("       --console             run gkrellmd on console (not as a service)\n"));
+	g_print(_("       --install             install gkrellmd service and exit\n"));
+	g_print(_("       --uninstall           uninstall gkrellmd service and exit\n"));
+	g_print(_("   -h, --help                display this help and exit\n"));
+	g_print(_("   -v, --version             output version information and exit\n"));
+	g_print(_("options (only for command '--console'):\n"));
+	g_print(_("   -u, --update-hz F         Monitor update frequency\n"));
+	g_print(_("   -m, --max-clients N       Number of simultaneous clients\n"));
+	g_print(_("   -A, --address A           Address of network interface to listen on\n"));
+	g_print(_("   -P, --port P              Server port to listen on\n"));
+	g_print(_("   -a, --allow-host host     Allow connections from specified hosts\n"));
+	g_print(_("   -c, --clear-hosts         Clears the current list of allowed hosts\n"));
+	g_print(_("       --io-timeout N        Close connection after N seconds of no I/O\n"));
+	g_print(_("       --reconnect-timeout N Try to connect every N seconds after\n"
+	           "                             a disconnect\n"));
+	g_print(_("   -p,  --plugin name        Enable a command line plugin\n"));
+	g_print(_("   -pe, --plugin-enable name Enable an installed plugin\n"));
+	g_print(_("       --plist               List plugins and exit\n"));
+	g_print(_("       --plog                Print plugin install log\n"));
+	g_print(  "       --logfile path        Enable logging to a file\n");
+	g_print(  "       --syslog              Enable logging to syslog\n");
+	g_print(_("   -V, --verbose             increases the verbosity of gkrellmd\n"));
+	g_print(_("   -d, --debug-level n       Turn debugging on for selective code sections.\n"));
+
+#else
+
+	g_print(_("usage: gkrellmd [options]\n"));
+	g_print(_("options:\n"));
+	g_print(_("   -u, --update-hz F         Monitor update frequency\n"));
+	g_print(_("   -m, --max-clients N       Number of simultaneous clients\n"));
+	g_print(_("   -A, --address A           Address of network interface to listen on\n"));
+	g_print(_("   -P, --port P              Server port to listen on\n"));
+	g_print(_("   -a, --allow-host host     Allow connections from specified hosts\n"));
+	g_print(_("   -c, --clear-hosts         Clears the current list of allowed hosts\n"));
+	g_print(_("       --io-timeout N        Close connection after N seconds of no I/O\n"));
+	g_print(_("       --reconnect-timeout N Try to connect every N seconds after\n"
 	         "                             a disconnect\n"));
-#if !defined(WIN32)
-	printf(_("       --mailbox path        Send local mailbox counts to gkrellm clients.\n"));
-	printf(_("   -d, --detach              Run in background and detach from terminal\n"));
-	printf(_("   -U, --user username       Change to this username after startup\n"));
-	printf(_("   -G, --group groupname     Change to this group after startup\n"));
+	g_print(_("       --mailbox path        Send local mailbox counts to gkrellm clients.\n"));
+	g_print(_("   -d, --detach              Run in background and detach from terminal\n"));
+	g_print(_("   -U, --user username       Change to this username after startup\n"));
+	g_print(_("   -G, --group groupname     Change to this group after startup\n"));
+	g_print(_("   -p,  --plugin name        Enable a command line plugin\n"));
+	g_print(_("   -pe, --plugin-enable name Enable an installed plugin\n"));
+	g_print(_("       --plist               List plugins and exit\n"));
+	g_print(_("       --plog                Print plugin install log\n"));
+	g_print(  "       --logfile path        Enable logging to a file\n");
+	g_print(  "       --syslog              Enable logging to the system syslog file\n");
+	g_print(_("       --pidfile path        Create a PID file\n"));
+	g_print(_("   -V, --verbose             increases the verbosity of gkrellmd\n"));
+	g_print(_("   -h, --help                display this help and exit\n"));
+	g_print(_("   -v, --version             output version information and exit\n"));
+
 #endif
-	printf(_("   -p,  --plugin name        Enable a command line plugin\n"));
-	printf(_("   -pe, --plugin-enable name Enable an installed plugin\n"));
-	printf(_("       --plist               List plugins and exit\n"));
-	printf(_("       --plog                Print plugin install log\n"));
-	printf(_("       --pidfile path        Create a PID file\n"));
-	printf(_("   -V, --verbose\n"));
-	printf(_("   -v, --version\n"));
 	}
 
 static void
 get_args(gint argc, gchar **argv)
 	{
-	gchar	*opt, *arg;
+	gchar	*s;
 	gint	i, r;
 
 	for (i = 1; i < argc; ++i)
 		{
-		opt = argv[i];
-		arg = argv[i + 1];
-		if (*opt == '-')
+		s = argv[i];
+		if (*s == '-')
 			{
-			++opt;
-			if (*opt == '-')
-				++opt;
+			++s;
+			if (*s == '-')
+				++s;
 			}
-		if (!strcmp(opt, "verbose") || !strcmp(opt, "V"))
-			{
+
+		if (!strcmp(s, "verbose") || !strcmp(s, "V"))
 			_GK.verbose += 1;
-			continue;
-			}
-		if (!strcmp(opt, "plist"))
-			{
+		else if (!strcmp(s, "plist"))
 			_GK.list_plugins = TRUE;
-			continue;
-			}
-		if (!strcmp(opt, "plog"))
-			{
+		else if (!strcmp(s, "plog"))
 			_GK.log_plugins = TRUE;
-			continue;
-			}
-		if (!strcmp(opt, "without-libsensors"))
-			{
+#if !defined(WIN32)
+		else if (!strcmp(s, "without-libsensors"))
 			_GK.without_libsensors = TRUE;
-			continue;
-			}
-		else if (!strcmp(opt, "help") || !strcmp(opt, "h"))
-			{
-			usage();
-			exit(0);
-			}
-		else if (!strcmp(opt, "version") || !strcmp(opt, "v"))
-			{
-			printf("gkrellmd %d.%d.%d%s\n", GKRELLMD_VERSION_MAJOR,
-					GKRELLMD_VERSION_MINOR, GKRELLMD_VERSION_REV,
-					GKRELLMD_EXTRAVERSION);
-			exit(0);
-			}
-		else if (   i < argc
-				 && ((r = parse_config(opt, (i < argc - 1) ? arg : NULL)) >= 0)
+#endif /* !WIN32 */
+		else if ((!strcmp(s, "debug-level") || !strcmp(s, "d")) && i < argc - 1)
+			_GK.debug_level = (gint) strtoul(argv[++i], NULL, 0);
+		else if ( i < argc
+				 && ((r = parse_config(s, (i < argc - 1) ? argv[i+1] : NULL)) >= 0)
 				)
 			{
 			i += r;
-			continue;
 			}
-
-		printf(_("Bad arg: %s\n"), argv[i]);
-		usage();
-		exit(0);
-		}
+		else
+			{
+			g_print(_("Bad arg: %s\n"), argv[i]);
+			usage();
+			exit(0);
+			}
+		} // for()
 	}
 
+
 static int *
 socksetup(int af)
 	{
@@ -750,7 +918,7 @@ socksetup(int af)
 		}
 	if (error)
 		{
-		fprintf(stderr, "gkrellmd %s\n", gai_strerror(error));
+		g_warning("gkrellmd %s\n", gai_strerror(error));
 		return NULL;
 		}
 #else
@@ -780,8 +948,7 @@ socksetup(int af)
 	socks = malloc((maxs + 1) * sizeof(int));
 	if (!socks)
 		{
-		fprintf(stderr,
-			"gkrellmd couldn't allocate memory for sockets\n");
+		g_warning("Could not allocate memory for sockets\n");
 		return NULL;
 		}
 
@@ -797,14 +964,14 @@ socksetup(int af)
 		*/
 		if (1)
 			{
-#ifdef WIN32
+#if defined(WIN32)
 			const char on = 1;
 #else
 			const int on = 1;
 #endif
 			if (setsockopt(*s, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0)
 				{
-				fprintf(stderr, "gkrellmd setsockopt (SO_REUSEADDR) failed\n");
+				g_warning("gkrellmd: setsockopt (SO_REUSEADDR) failed\n");
 				close(*s);
 				continue;
 				}
@@ -817,7 +984,7 @@ socksetup(int af)
 
 			if (setsockopt(*s, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on)) < 0)
 				{
-				fprintf(stderr, "gkrellmd setsockopt (IPV6_V6ONLY) failed\n");
+				g_warning("gkrellmd: setsockopt (IPV6_V6ONLY) failed\n");
 				close(*s);
 				continue;
 				}
@@ -839,13 +1006,14 @@ socksetup(int af)
 
 	if (*socks == 0)
 		{
-		fprintf(stderr, "gkrellmd couldn't bind to any socket\n");
+		g_warning("Could not bind to any socket\n");
 		free(socks);
 		return NULL;
 		}
 	return socks;
 	}
 
+#if !defined(WIN32)
 /* XXX: Recent glibc seems to have daemon(), too. */
 #if defined(BSD4_4)
 #define HAVE_DAEMON
@@ -859,23 +1027,21 @@ socksetup(int af)
 #define _PATH_DEVNULL	"/dev/null"
 #endif
 
-static void
+static gboolean
 detach_from_terminal(void)
 	{
-#if !defined(WIN32)
 #if !defined(HAVE_DAEMON)
 	gint	i, fd;
-#endif
+#endif /* HAVE_DAEMON */
 
 	if (getppid() == 1)	 /* already a daemon */
-		return;
+		return TRUE;
 
 #if defined(HAVE_DAEMON)
 	if (daemon(0, 0))
 		{
-		fprintf(stderr, "gkrellmd detach failed: %s\n", strerror(errno));
-		remove_pidfile();
-		exit(1);
+		g_warning("Detach failed: %s\n", strerror(errno));
+		return FALSE;
 		}
 #else
 	i = fork();
@@ -884,9 +1050,8 @@ detach_from_terminal(void)
 
 	if (i < 0 || setsid() == -1)		/* new session process group */
 		{
-		fprintf(stderr, "gkrellmd detach failed: %s\n", strerror(errno));
-		remove_pidfile();
-		exit(1);
+		g_warning("Detach failed: %s\n", strerror(errno));
+		return FALSE;
 		}
 
 	if ((fd = open(_PATH_DEVNULL, O_RDWR, 0)) != -1)
@@ -898,7 +1063,7 @@ detach_from_terminal(void)
 			close(fd);
 		}
 	chdir("/");
-#endif
+#endif /* HAVE_DAEMON */
 
 //	signal(SIGCHLD, SIG_IGN);
 	signal(SIGTSTP, SIG_IGN);
@@ -907,10 +1072,10 @@ detach_from_terminal(void)
 	signal(SIGHUP, SIG_IGN);
 #if !defined(MSG_NOSIGNAL)
 	signal(SIGPIPE, SIG_IGN);
-#endif
-#endif
+#endif /* MSG_NOSIGNAL */
+	return TRUE;
 	}
-
+#endif /* !defined(WIN32) */
 
 
 static void
@@ -928,19 +1093,9 @@ drop_privileges(void)
 	}
 
 
-#if defined(WIN32)
-int APIENTRY
-WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmd, int nCmdShow)
+static gint
+gkrellmd_run(gint argc, gchar **argv)
 	{
-	int		argc;
-	gchar	**argv;
-	gchar	*cmd;
-
-#else
-gint
-main(gint argc, gchar **argv)
-	{
-#endif // WIN32
 #ifdef HAVE_GETADDRINFO
 	struct sockaddr_storage client_addr;
 #else
@@ -952,52 +1107,49 @@ main(gint argc, gchar **argv)
 	size_t				addr_len;
 	gint				fd, server_fd, client_fd, i;
 #if defined(WIN32)
-	gulong			nbytes;
+	gulong				nbytes;
 #else
 	gint				nbytes;
-#endif // WIN32
+#endif /* defined(WIN32) */
 	gint				max_fd = -1;
 	gint				listen_fds = 0;
 	gint				interval, result;
 
-#if defined(WIN32)
-    createServerWindow(hInst);
-
-    // lpCmd does not contain the exe name, GetCommandLine() does :)
-	cmd = GetCommandLine();
-	g_shell_parse_argv(cmd, &argc, &argv, NULL);
-#endif // WIN32
-
-#ifdef ENABLE_NLS
-#ifdef LOCALEDIR
-	bindtextdomain(PACKAGE_D, LOCALEDIR);
-#endif
-	textdomain(PACKAGE_D);
-	bind_textdomain_codeset(PACKAGE_D, "UTF-8");
-#endif	/* ENABLE_NLS */
-
 	read_config();
 	get_args(argc, argv);
 
-	if (_GK.verbose)
-		printf("update_HZ=%d\n", _GK.update_HZ);
+	// first message that might get logged
+	g_message("Starting GKrellM Daemon %d.%d.%d%s\n", GKRELLMD_VERSION_MAJOR,
+			GKRELLMD_VERSION_MINOR, GKRELLMD_VERSION_REV,
+			GKRELLMD_EXTRAVERSION);
 
+	if (_GK.verbose)
+		g_print("update_HZ=%d\n", _GK.update_HZ);
+
+#if defined(WIN32)
+	if (!service_is_one)
+		{
+		signal(SIGTERM, cb_sigterm);
+		signal(SIGINT, cb_sigterm);
+		}
+#else
 	if (   detach_flag
 	    && !_GK.log_plugins && !_GK.list_plugins && _GK.debug_level == 0
 	   )
-		detach_from_terminal();
+		{
+		if (detach_from_terminal() == FALSE)
+			return 1;
+		}
 	else
 		{
 		signal(SIGTERM, cb_sigterm);
-#if !defined(WIN32)
 		signal(SIGQUIT, cb_sigterm);
 		signal(SIGTSTP, SIG_IGN);
-#endif
 		signal(SIGINT, cb_sigterm);
 		}
+#endif /* defined(WIN32) */
 
 	make_pidfile();
-
 	gkrellm_sys_main_init();
 	drop_privileges();
 
@@ -1020,9 +1172,9 @@ main(gint argc, gchar **argv)
 	_GK.server_fd = socksetup(PF_UNSPEC);
 	if (_GK.server_fd == NULL)
 		{
-		fprintf(stderr, "gkrellmd socket() failed: %s\n", strerror(errno));
-		remove_pidfile();
-		exit(1);
+		g_warning("socket() failed: %s\n", strerror(errno));
+		gkrellmd_cleanup();
+		return 1;
 		}
 
 	/* Listen on the socket so a client gkrellm can connect.
@@ -1042,28 +1194,25 @@ main(gint argc, gchar **argv)
 		}
 	if (listen_fds <= 0)
 		{
-		fprintf(stderr, "gkrellmd listen() failed: %s\n", strerror(errno));
-		remove_pidfile();
-		exit(1);
+		g_warning("listen() failed: %s\n", strerror(errno));
+		gkrellmd_cleanup();
+		return 1;
 		}
 
 	interval = 1000000 / _GK.update_HZ;
 
+	gkrellm_debug(DEBUG_SERVER, "Entering main event loop\n");
+	// main event loop
 #if defined(WIN32)
-	while (!done)
-        {
-            {
-                MSG msg;
-		while (PeekMessage(&msg, (HWND) NULL, 0, 0, PM_REMOVE))
-			{
-                    TranslateMessage(&msg); 
-                    DispatchMessage(&msg); 
-                }
-            }
+	/* endless loop if:
+	   - we're a service and our service_running flag is TRUE
+	   - we're a console-app (--console argument passed at startup
+	*/
+	while(service_running == TRUE || service_is_one == FALSE)
 #else
-    while(1)
-		{
+	while(1)
 #endif
+		{
 		test_fds = read_fds;
 #ifdef HAVE_GETADDRINFO
 		addr_len = sizeof(struct sockaddr_storage);
@@ -1072,43 +1221,50 @@ main(gint argc, gchar **argv)
 #endif
 		tv.tv_usec = interval;
 		tv.tv_sec = 0;
-		if ((result = select(max_fd + 1, &test_fds, NULL, NULL, &tv) == -1))
+
+		result = select(max_fd + 1, &test_fds, NULL, NULL, &tv);
+		if (result == -1)
 			{
 			if (errno == EINTR)
 				continue;
-			fprintf(stderr, "gkrellmd select() failed: %s\n", strerror(errno));
-			remove_pidfile();
-			exit(1);
+			g_warning("select() failed: %s\n", strerror(errno));
+			gkrellmd_cleanup();
+			return 1;
 			}
+
 #if 0	/* BUG, result is 0 when test_fds has a set fd!! */
-		if (result == 0)	
+		if (result == 0)
 			{
 			gkrellmd_update_monitors();
 			continue;
 			}
 #endif
+
 		for (fd = 0; fd <= max_fd; ++fd)
 			{
 			if (!FD_ISSET(fd, &test_fds))
 				continue;
 			server_fd = -1;
 			for (i = 1; i <= _GK.server_fd[0]; ++i)
+				{
 				if (fd == _GK.server_fd[i])
 					{
 					server_fd = fd;
 					break;
 					}
+				}
 			if (server_fd >= 0)
 				{
+				gkrellm_debug(DEBUG_SERVER, "Calling accept() for new client connection\n");
 				client_fd = accept(server_fd,
 						(struct sockaddr *) &client_addr,
 						(socklen_t *) (void *)&addr_len);
 				if (client_fd == -1)
 					{
-					fprintf(stderr, "gkrellmd accept() failed: %s\n",
+					g_warning("accept() failed: %s\n",
 							strerror(errno));
-					remove_pidfile();
-					exit(1);
+					gkrellmd_cleanup();
+					return 1;
 					}
 				if (client_fd > max_fd)
 					max_fd = client_fd;
@@ -1121,14 +1277,19 @@ main(gint argc, gchar **argv)
 					}
 				FD_SET(client_fd, &read_fds);
 				gkrellmd_serve_setup(client);
-				if (_GK.verbose)
-					printf("gkrellmd accepted client: %s:%u\n",
-						client->hostname,
-						ntohs(((struct sockaddr_in *)&client_addr)->sin_port));
+
+				g_message(_("Accepted client %s:%u\n"),
+					client->hostname,
+					ntohs(((struct sockaddr_in *)&client_addr)->sin_port));
 				}
 			else
 				{
+				gkrellm_debug(DEBUG_SERVER, "Reading data from client connection\n");
+#if defined(WIN32)
+				ioctlsocket(fd, FIONREAD, &nbytes);
+#else
 				ioctl(fd, FIONREAD, &nbytes);
+#endif
 				if (nbytes == 0)
 					{
 					remove_client(fd);
@@ -1139,13 +1300,291 @@ main(gint argc, gchar **argv)
 				}
 			}
 		gkrellmd_update_monitors();
-		}
-		
-#ifdef WIN32
-	deleteServerWindow();
-#endif
+		} // while(1)
 	return 0;
-	} // main()/WinMain()
+	} // gkrellmd_main()
+
+
+#if defined(WIN32)
+static void service_update_status(DWORD newState)
+{
+	service_status.dwCurrentState = newState;
+	SetServiceStatus(service_status_handle, &service_status);
+}
+
+void WINAPI service_control_handler(DWORD controlCode)
+	{
+	switch (controlCode)
+		{
+		case SERVICE_CONTROL_SHUTDOWN:
+		case SERVICE_CONTROL_STOP:
+			service_update_status(SERVICE_STOP_PENDING);
+			service_running = FALSE;
+			return;
+
+		default:
+			break;
+		}
+	}
+
+void WINAPI service_main(DWORD argc, WCHAR* argv[])
+	{
+	gchar **argv_utf8;
+	DWORD i;
+
+	/* Init service status */
+	service_status.dwServiceType = SERVICE_WIN32;
+	service_status.dwCurrentState = SERVICE_STOPPED;
+	service_status.dwControlsAccepted = 0;
+	service_status.dwWin32ExitCode = NO_ERROR;
+	service_status.dwServiceSpecificExitCode = NO_ERROR;
+	service_status.dwCheckPoint = 0;
+	service_status.dwWaitHint = 0;
+
+	service_status_handle = RegisterServiceCtrlHandlerW(service_name, service_control_handler);
+	if (service_status_handle)
+		{
+		// convert all strings in argv pointer array from utf16 to utf8
+		argv_utf8 = g_malloc(argc * sizeof(gchar *));
+		for (i = 0; i < argc; i++)
+			argv_utf8[i] = g_utf16_to_utf8(argv[i], -1, NULL, NULL, NULL);
+
+		// service is starting
+		service_update_status(SERVICE_START_PENDING);
+
+		// service is running
+		service_status.dwControlsAccepted |= (SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN);
+		service_update_status(SERVICE_RUNNING);
+
+		service_running = TRUE;
+
+		// gkrellmd_main returns on error or as soon as
+		// service_running is FALSE (see service_control_handler())
+		gkrellmd_run(argc, argv_utf8);
+
+		// service was stopped
+		service_update_status(SERVICE_STOP_PENDING);
+
+		// services are not stopped via process signals so we have to
+		// clean up like in cb_sigterm() but without calling exit()!
+		g_message("GKrellM Daemon %d.%d.%d%s: Exiting normally\n",
+				GKRELLMD_VERSION_MAJOR, GKRELLMD_VERSION_MINOR,
+				GKRELLMD_VERSION_REV, GKRELLMD_EXTRAVERSION);
+		gkrellmd_cleanup();
+
+		// free all strings in pointer array and free the array itself
+		for (i = 0; i < argc; i++)
+			g_free(argv_utf8[i]);
+		g_free(argv_utf8);
+
+		// service is now stopped
+		service_status.dwControlsAccepted &= ~(SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN);
+		service_update_status(SERVICE_STOPPED);
+
+		// process is automatically terminated by windows now
+		}
+	}
+
+void service_run()
+	{
+	SERVICE_TABLE_ENTRYW service_table[] =
+		{ {service_name, service_main}, { 0, 0 } };
+	service_is_one = TRUE;
+	// Blocking system call, will return if service is not needed anymore
+	StartServiceCtrlDispatcherW(service_table);
+	}
+
+static gboolean service_wait_for_stop(SC_HANDLE serviceHandle)
+	{
+	static const gulong waitTimeoutSec = 30;
+	SERVICE_STATUS status;
+	GTimer *waitTimer = NULL;
+	gboolean ret = FALSE;
+
+	if (!QueryServiceStatus(serviceHandle, &status))
+		{
+		g_warning("Could not query status of %ls (%ld)\n", service_display_name, GetLastError());
+		return FALSE;
+		}
+	waitTimer = g_timer_new(); /* create and start */
+	while (status.dwCurrentState == SERVICE_STOP_PENDING)
+		{
+		g_usleep(status.dwWaitHint * 1000);
+		if (!QueryServiceStatus(serviceHandle, &status))
+			{
+			g_warning("Could not query status of %ls (%ld)\n", service_display_name, GetLastError());
+			ret = FALSE;
+			break;
+			}
+		if (status.dwCurrentState == SERVICE_STOPPED)
+			{
+			ret = TRUE;
+			break;
+			}
+		if (g_timer_elapsed(waitTimer, NULL) > waitTimeoutSec)
+			{
+			g_warning("Stopping %ls timed out\n", service_display_name);
+			ret = FALSE;
+			break;
+			}
+		} /*while*/
+	g_timer_destroy(waitTimer);
+	return ret;
+	}
+
+
+static gboolean service_stop(SC_HANDLE serviceHandle)
+	{
+	SERVICE_STATUS svcStatus;
+	if (!QueryServiceStatus(serviceHandle, &svcStatus))
+		{
+		g_warning("Could not query status of %ls (%ld)\n", service_display_name, GetLastError());
+		return FALSE;
+		}
+	/* service not running at all, just return that stopping worked out */
+	if (svcStatus.dwCurrentState == SERVICE_STOPPED)
+		{
+		g_print(_("%ls already stopped\n"), service_display_name);
+		return TRUE;
+		}
+	/* service already stopping, just wait for its exit */
+	if (svcStatus.dwCurrentState == SERVICE_STOP_PENDING)
+		{
+		return service_wait_for_stop(serviceHandle);
+		}
+	/* Service is running, let's stop it */
+	if (!ControlService(serviceHandle, SERVICE_CONTROL_STOP, &svcStatus))
+		{
+		g_warning(_("Could not stop %ls (%ld)\n"), service_display_name, GetLastError());
+		return FALSE;
+		}
+	// Wait for the service to stop.
+	if (svcStatus.dwCurrentState == SERVICE_STOP_PENDING)
+		{
+		return service_wait_for_stop(serviceHandle);
+		}
+	return TRUE;
+	}
+
+
+static gboolean service_install()
+	{
+	WCHAR path[_MAX_PATH + 1];
+	SC_HANDLE scmHandle;
+	SC_HANDLE svcHandle;
+	DWORD err;
+
+	g_print(_("Installing %ls...\n"), service_display_name);
+
+	if (GetModuleFileNameW(0, path, sizeof(path)/sizeof(path[0])) < 1)
+	{
+		g_warning("Could not determine path to gkrellmd service binary, error 0x%ld\n", GetLastError());
+		return FALSE;
+	}
+
+	scmHandle = OpenSCManagerW(NULL, NULL, SC_MANAGER_CREATE_SERVICE);
+	if (!scmHandle)
+		{
+		err = GetLastError();
+		if (err == ERROR_ACCESS_DENIED)
+			g_warning("Could not connect to service manager, access denied\n");
+		else
+			g_warning("Could not connect to service manager, error 0x%lXd\n", err);
+		return FALSE;
+		}
+
+	svcHandle = CreateServiceW(scmHandle, service_name, service_display_name,
+			SERVICE_ALL_ACCESS, SERVICE_WIN32_OWN_PROCESS, SERVICE_AUTO_START,
+			SERVICE_ERROR_NORMAL, path, 0, 0, 0, 0, 0);
+	if (!svcHandle)
+		{
+		err = GetLastError();
+		if (err == ERROR_ACCESS_DENIED)
+			g_warning("Could not install %ls, access denied\n", service_display_name);
+		else if (err == ERROR_SERVICE_EXISTS || err == ERROR_DUPLICATE_SERVICE_NAME)
+			g_warning("Could not install %ls, a service of the same name already exists\n", service_display_name);
+		else
+			g_warning("Could not install %ls, error 0x%lX\n", service_display_name, err);
+		CloseServiceHandle(scmHandle);
+		return FALSE;
+		}
+	else
+		{
+		g_print(_("%ls has been installed.\n"), service_display_name);
+		}
+
+	g_print(_("Starting %ls...\n"), service_display_name);
+	if (!StartServiceW(svcHandle, 0, NULL))
+		{
+		err = GetLastError();
+		if (err == ERROR_ACCESS_DENIED)
+			g_warning("Could not start %ls, access denied\n", service_display_name);
+		else
+			g_warning("Could not start %ls, error 0x%lX\n", service_display_name, err);
+		}
+	else
+		{
+		g_print(_("%ls has been started.\n"), service_display_name);
+		}
+
+	CloseServiceHandle(svcHandle);
+	CloseServiceHandle(scmHandle);
+	return TRUE;
+	} /* service_install() */
+
+
+static gboolean service_uninstall()
+	{
+	SC_HANDLE scmHandle;
+	SC_HANDLE svcHandle;
+	BOOL delRet = FALSE;
+	gchar *errmsg;
+
+	g_print(_("Uninstalling %ls...\n"), service_display_name);
+
+	scmHandle = OpenSCManagerW(NULL, NULL, SC_MANAGER_CONNECT);
+	if (!scmHandle)
+		{
+		errmsg = g_win32_error_message(GetLastError());
+		g_warning("Could not connect to service manager: %s\n", errmsg);
+		g_free(errmsg);
+
+		return FALSE;
+		}
+
+	svcHandle = OpenServiceW(scmHandle, service_name,
+			SERVICE_STOP | SERVICE_QUERY_STATUS | DELETE);
+	if (!svcHandle)
+		{
+		errmsg = g_win32_error_message(GetLastError());
+		g_warning("Could not open %ls: %s\n", service_display_name, errmsg);
+		g_free(errmsg);
+		}
+	else
+		{
+		// handle to gkrellm service aquired, now stop and uninstall it
+		if (service_stop(svcHandle))
+			{
+			delRet = DeleteService(svcHandle);
+			if (!delRet)
+				{
+				errmsg = g_win32_error_message(GetLastError());
+				g_warning("Could not uninstall %ls: %s\n",
+						service_display_name, errmsg);
+				g_free(errmsg);
+				}
+			else
+				{
+				g_print(_("%ls has been uninstalled.\n"), service_display_name);
+				}
+			}
+		CloseServiceHandle(svcHandle);
+		}
+	CloseServiceHandle(scmHandle);
+
+	return delRet ? TRUE : FALSE;
+	} /* service_uninstall() */
+#endif /* defined(WIN32) */
 
 
 GkrellmdTicks *
@@ -1154,8 +1593,112 @@ gkrellmd_ticks(void)
 	return &GK;
 	}
 
+
 gint
 gkrellmd_get_timer_ticks(void)
 	{
 	return GK.timer_ticks;
 	}
+
+
+int main(int argc, char* argv[])
+	{
+	int i;
+	char *opt;
+
+#ifdef ENABLE_NLS
+#ifdef LOCALEDIR
+#if defined(WIN32)
+	/*
+	Prepend app install path to relative locale dir, don't rely on CWD being correct
+	*/
+	if (!G_IS_DIR_SEPARATOR(LOCALEDIR[0]))
+		{
+ 		gchar* locale_dir;
+		locale_dir = g_win32_get_package_installation_subdirectory(NULL, NULL, LOCALEDIR);
+		if (locale_dir != NULL)
+			{
+			bindtextdomain(PACKAGE_D, locale_dir);
+			g_free(locale_dir);
+			}
+		}
+	else
+		{
+		bindtextdomain(PACKAGE_D, LOCALEDIR);
+		}
+#else
+	bindtextdomain(PACKAGE_D, LOCALEDIR);
+#endif /* !WIN32 */
+#endif /* LOCALEDIR */
+	textdomain(PACKAGE_D);
+	bind_textdomain_codeset(PACKAGE_D, "UTF-8");
+#endif	/* ENABLE_NLS */
+
+	// Init logging-chain
+	gkrellm_log_init();
+
+	/* Parse arguments for actions that exit gkrellmd immediately */
+	for (i = 1; i < argc; ++i)
+		{
+		opt = argv[i];
+		if (*opt == '-')
+			{
+			++opt;
+			if (*opt == '-')
+				++opt;
+			}
+
+		if (!strcmp(opt, "help") || !strcmp(opt, "h"))
+			{
+			usage();
+			return 0;
+			}
+		else if (!strcmp(opt, "version") || !strcmp(opt, "v"))
+			{
+			g_print("gkrellmd %d.%d.%d%s\n", GKRELLMD_VERSION_MAJOR,
+					GKRELLMD_VERSION_MINOR, GKRELLMD_VERSION_REV,
+					GKRELLMD_EXTRAVERSION);
+			return 0;
+			}
+#if defined(WIN32)
+		else if (!strcmp(opt, "install"))
+			{
+			return (service_install() ? 1 : 0);
+			}
+		else if (!strcmp(opt, "uninstall"))
+			{
+			return (service_uninstall() ? 1 : 0);
+			}
+		else if (!strcmp(opt, "console"))
+			{
+			/*
+			Special case for windows: run gkrellmd on console and not as
+			a service. This is helpful for debugging purposes.
+			*/
+			int retVal;
+			int newArgc = 0;
+			char **newArgv = malloc((argc - 1) * sizeof(char *));
+			int j;
+			for (j = 0; j < argc; ++j)
+			{
+				/* filter out option "--console" */
+				if (j == i)
+					continue;
+				newArgv[newArgc++] = argv[j];
+			}
+			retVal = gkrellmd_run(newArgc, newArgv);
+			free(newArgv);
+			return retVal;
+			}
+#endif /* defined(WIN32) */
+		}
+
+#if defined(WIN32)
+	// win32: register service and wait for the service to be started/stopped
+	service_run();
+	return 0;
+#else
+	// Unix: just enter main loop
+	return gkrellmd_run(argc, argv);
+#endif
+	}
