@@ -1,22 +1,22 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
 #include "gkrellmd.h"
@@ -24,9 +24,6 @@
 
 #include "../src/gkrellm-sysdeps.h"
 
-static gboolean (*mbmon_check_func)();
-static gint gkrellm_connect_to(gchar *, gint);
-
 #if defined(__linux__)
 #include "../src/sysdeps/linux.c"
 #include "../src/sysdeps/sensors-common.c"
@@ -43,10 +40,17 @@ static gint gkrellm_connect_to(gchar *, gint);
 #include "../src/sysdeps/sensors-common.c"
 #endif
 
+#if defined(__DragonFly__)
+#include "../src/sysdeps/dragonfly.c"
+#include "../src/sysdeps/bsd-common.c"
+#include "../src/sysdeps/sensors-common.c"
+#endif
+
 #if defined(__NetBSD__)
 #include "../src/sysdeps/netbsd.c"
 #include "../src/sysdeps/bsd-net-open.c"
 #include "../src/sysdeps/bsd-common.c"
+#include "../src/sysdeps/sensors-common.c"
 #endif
 
 #if defined(__OpenBSD__)
@@ -64,7 +68,13 @@ static gint gkrellm_connect_to(gchar *, gint);
 #include "../src/sysdeps/gtop.c"
 #endif
 
+#if defined(WIN32)
+#include "../src/sysdeps/win32.c"
+#endif
+
+#if !defined(WIN32)
 #include <sys/utsname.h>
+#endif
 
 gchar *
 gkrellm_sys_get_host_name(void)
@@ -78,6 +88,7 @@ gkrellm_sys_get_host_name(void)
 	return buf;
 	}
 
+#if !defined(WIN32)
 gchar *
 gkrellm_sys_get_system_name(void)
 	{
@@ -90,7 +101,7 @@ gkrellm_sys_get_system_name(void)
 		sname = g_strdup("unknown name");
 	return sname;
 	}
-
+#endif
 
   /* Remove embedded "-i2c-" or "-isa-" from lm_sensors chip names so
   |  there can be a chance for config name sysfs compatibility.  This function
@@ -123,7 +134,7 @@ gkrellm_sensors_linux_name_fix(gchar *id_name)
 	}
 
 #ifdef SENSORS_COMMON
-static gint
+gint
 gkrellm_connect_to(gchar *server, gint server_port)
 	{
 	gint		fd	= -1;
@@ -134,7 +145,7 @@ gkrellm_connect_to(gchar *server, gint server_port)
 #else
 	struct hostent	*addr;
 	struct sockaddr_in s;
-#endif
+#endif // HAVE_GETADDRINFO
 
 #ifdef HAVE_GETADDRINFO
 	snprintf (portnumber, sizeof(portnumber), "%d", server_port);
@@ -160,7 +171,7 @@ gkrellm_connect_to(gchar *server, gint server_port)
 		closesocket(fd);
 #else
 		close(fd);
-#endif
+#endif // WIN32
 		fd = -1;
 		}
 	freeaddrinfo(res0);
@@ -183,15 +194,15 @@ gkrellm_connect_to(gchar *server, gint server_port)
 				closesocket(fd);
 #else
 				close(fd);
-#endif
+#endif // WIN32
 				fd = -1;
 				}
 			}
 		}
-#endif
+#endif // HAVE_GETADDRINFO
 	if (fd < 0)
 		return -1;
 
 	return fd;
 	}
-#endif
+#endif // SENSORS_COMMON
