@@ -1,22 +1,34 @@
-/*
-|  Copyright (C) 1999-2006 Bill Wilson
+/* GKrellM
+|  Copyright (C) 1999-2008 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -1209,11 +1221,21 @@ create_battery_tab(GtkWidget *tab_vbox)
 
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
 
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Launch Commands"),
+				4, 0, TRUE);
+	table = gkrellm_gtk_launcher_table_new(vbox1, 1);
+	gkrellm_gtk_config_launcher(table, 0, &launch_entry, &tooltip_entry, 
+					_("Battery"), &launch);
+	g_signal_connect(G_OBJECT(launch_entry), "changed",
+			G_CALLBACK(cb_launch_entry), NULL);
+	g_signal_connect(G_OBJECT(tooltip_entry), "changed",
+			G_CALLBACK(cb_launch_entry), NULL);
+
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 8);
+	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 8);
 	gkrellm_gtk_alert_button(hbox, NULL, FALSE, FALSE, 4, TRUE,
 				cb_set_alert, NULL);
-
 	if (battery_list)
 		{
 		bat = (Battery *) battery_list->data;
@@ -1224,15 +1246,6 @@ create_battery_tab(GtkWidget *tab_vbox)
 					_("Alerts check for percent capacity remaining."));
 		}
 
-	vbox1 = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
-			4, FALSE, 0, 2);
-	table = gkrellm_gtk_launcher_table_new(vbox1, 1);
-	gkrellm_gtk_config_launcher(table, 0, &launch_entry, &tooltip_entry, 
-					_("Battery"), &launch);
-	g_signal_connect(G_OBJECT(launch_entry), "changed",
-			G_CALLBACK(cb_launch_entry), NULL);
-	g_signal_connect(G_OBJECT(tooltip_entry), "changed",
-			G_CALLBACK(cb_launch_entry), NULL);
 
 /* --Info tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Info"));
