@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2008 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,6 +17,18 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 
@@ -34,8 +46,8 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 #else
-#define uint32_t u_long
 #include <winsock2.h>
+#include <ws2tcpip.h>
 #endif
 
 #include <errno.h>
@@ -1481,6 +1493,8 @@ client_plugin_func(gchar *line)
 	for (list = client_plugin_serve_data_list; list; list = list->next)
 		{
 		plug = (ClientPlugin *) list->data;
+		if (!plug->key_name)
+			continue;
 		n = strlen(plug->key_name);
 		s = line + n;
 		if (*s == '>' && !strncmp(plug->key_name, line, n))
@@ -1512,7 +1526,7 @@ client_plugin_initial_update(ClientPlugin *plug)
 			}
 		else if (func)
 			(*func)(line);
-		}	
+		}
 	g_free(serve_name);
 	}
 
@@ -1539,7 +1553,7 @@ client_send_to_server(gchar *buf)
 	{
 	gint	n;
 
-	if (!server_alive || client_fd < 0)
+	if (!server_alive || client_fd < 0 || !buf)
 		return FALSE;
 #if defined(MSG_NOSIGNAL)
 	n = send(client_fd, buf, strlen(buf), MSG_NOSIGNAL);
@@ -1681,7 +1695,7 @@ process_server_line(KeyTable *table, gint table_size, gchar *line)
   /* Read setup info from gkrellmd server.  Stuff needed before the
   |  client_init calls must be read here.
   */
-static void
+static gboolean
 read_server_setup(gint fd)
 	{
 	gchar			buf[256];
@@ -1696,9 +1710,13 @@ read_server_setup(gint fd)
 
 	gkrellm_free_glist_and_data(&client_plugin_setup_line_list);
 
+    gint rs;
+
 	while (1)
 		{
-		getline(fd, buf, sizeof(buf));
+		rs=getline(fd, buf, sizeof(buf));
+        if(rs<0)
+            return FALSE;
 		if (!strcmp(buf, "</gkrellmd_setup>"))
 			break;
 		process_server_line(&setup_table[0], table_size, buf);
@@ -1714,12 +1732,15 @@ read_server_setup(gint fd)
 	table_size = sizeof(update_table) / sizeof(KeyTable);
 	while (1)
 		{
-		getline(fd, buf, sizeof(buf));
+		rs=getline(fd, buf, sizeof(buf));
+        if(rs<0)
+            return FALSE;
 		if (!strcmp(buf, "</initial_update>"))
 			break;
 		process_server_line(&update_table[0], table_size, buf);
 		}
 	setup_done = TRUE;
+    return TRUE;
 	}
 
 void
@@ -1853,13 +1874,21 @@ gboolean
 gkrellm_client_mode_connect(void)
 	{
 	gchar	buf[128];
+	GtkWidget* dlg;
 
 	if (_GK.server_port == 0)
 		_GK.server_port = GKRELLMD_SERVER_PORT;
 
 	client_fd = gkrellm_connect_to(_GK.server, _GK.server_port);
 	if (client_fd < 0) {
-		printf("%s\n", _("Unable to connect."));
+		g_warning("%s\n", _("Unable to connect."));
+
+		dlg = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
+				GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+				"Unable to connect to server '%s:%d'",
+				_GK.server, _GK.server_port);
+		gtk_dialog_run(GTK_DIALOG(dlg));
+		gtk_widget_destroy(dlg);
 		return FALSE;
 	}
 
@@ -1870,7 +1899,18 @@ gkrellm_client_mode_connect(void)
 
 	/* Initial setup lines from server are read in blocking mode.
 	*/
-	read_server_setup(client_fd);
+	if(!read_server_setup(client_fd)){
+
+		close(client_fd);
+
+		dlg = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
+				GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+				"Initial setup with server '%s:%d' failed.",
+				_GK.server, _GK.server_port);
+		gtk_dialog_run(GTK_DIALOG(dlg));
+		gtk_widget_destroy(dlg);
+        return FALSE;
+    }
 
 	/* Extra stuff not handled in read_server_setup()
 	*/
