@@ -1,33 +1,60 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2008 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details. Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
 #include "gkrellm-private.h"
 
 #if defined(WIN32)
-#define	STRFTIME_24_HOUR	"%#H:%M"
-#define STRFTIME_12_HOUR	"%#I:%M"
+
+#define DEFAULT_CLOCK_FORMAT "%#I:%M <span foreground=\"$A\"><small>%S</small></span>"
+#define ALT1_CLOCK_FORMAT "%#I:%M <span foreground=\"$A\"><small>%p</small></span>"
+#define ALT2_CLOCK_FORMAT "%#H:%M <span foreground=\"$A\"><small>%S</small></span>"
+#define DEFAULT_CAL_FORMAT "%a <span foreground=\"$A\"><big><big>%#d</big></big></span> %b"
+#define ALT1_CAL_FORMAT "<big>%a %b <span foreground=\"$A\">%#d</span></big>"
+#define ALT2_CAL_FORMAT "%a <span foreground=\"cyan2\"><span font_desc=\"16.5\"><i>%#d</i></span></span> %b"
+
 #else
-#define	STRFTIME_24_HOUR	"%k:%M"
-#define STRFTIME_12_HOUR	"%l:%M"
+
+#define DEFAULT_CLOCK_FORMAT \
+		_("%l:%M <span foreground=\"$A\"><small>%S</small></span>")
+#define ALT1_CLOCK_FORMAT "%l:%M <span foreground=\"$A\"><small>%p</small></span>"
+#define ALT2_CLOCK_FORMAT "%k:%M <span foreground=\"$A\"><small>%S</small></span>"
+
+#define DEFAULT_CAL_FORMAT \
+		_("%a <span foreground=\"$A\"><big><big>%e</big></big></span> %b")
+#define ALT1_CAL_FORMAT  "<big>%a %b <span foreground=\"$A\">%e</span></big>"
+#define ALT2_CAL_FORMAT "%a <span foreground=\"cyan2\"><span font_desc=\"16.5\"><i>%e</i></span></span> %b"
+
 #endif
 
 
@@ -39,41 +66,37 @@ static GkrellmPanel
 				*pclock,
 				*pcal;
 
+static GtkWidget
+				*cal_vbox,
+				*clock_vbox;
+
 static GkrellmDecal
-				*d_wday,
-				*d_mday,
-				*d_month,
-				*d_clock,
-				*d_seconds;
+				*d_cal,
+				*d_clock;
 
 
 static GkrellmLauncher
 				clock_launch,
 				cal_launch;
 
-typedef struct
-	{
-	GkrellmTextstyle ts;
-	gint		width,
-				height,
-				baseline,
-				y_ink;
-	}
-	Extents;
-
 static gboolean	cal_enable,
 				clock_enable,
-				clock_12hr_sec_enable,
-				clock_24hr_sec_enable,
-				clock_24hr_mode,
 				loop_chime_enable;
 
-static gint		x_clock_seconds,
-				x_clock_no_seconds;
+
+static gchar	*cal_format,
+				*clock_format;
+
+static gchar	*cal_alt_color_string,
+				*clock_alt_color_string;
+
+static gint		clock_style_id,
+				cal_style_id;
 
 
 static gchar	*hour_chime_command,
 				*quarter_chime_command;
+static gboolean	chime_block;
 
 typedef struct
 	{
@@ -82,32 +105,17 @@ typedef struct
 	}
 	ChimeData;
 
-static Extents	wday_extents,
-				mday_extents,
-				month_extents,
-				time_extents,
-				sec_extents,
-				ampm_extents;
-
-static gint		clock_style_id,
-				cal_style_id;
-static gboolean	chime_block;
 
 struct tm		gkrellm_current_tm;
 
 
+
 struct tm *
 gkrellm_get_current_time(void)
 	{
 	return &gkrellm_current_tm;
 	}
 
-static void
-string_extents(gchar *text, Extents *ext)
-	{
-	gkrellm_text_extents(ext->ts.font, text, strlen(text),
-			&ext->width, &ext->height, &ext->baseline, &ext->y_ink);
-	}
 
 static gpointer
 chime_func(gpointer data)
@@ -115,9 +123,9 @@ chime_func(gpointer data)
 	ChimeData	*chime = (ChimeData *)data;
 	gint		counter;
 
-	if (strlen(chime->command)) 
+	if (strlen(chime->command))
 		{
-		if (chime->count > 12) 
+		if (chime->count > 12)
 			chime->count -= 12;
 
 		for (counter = 0; counter < chime -> count; counter ++)
@@ -129,157 +137,81 @@ chime_func(gpointer data)
 	return NULL;
 	}
 
+static void
+get_color_name(GdkColor *color, gchar **color_string)
+	{
+	gchar	*cstring;
+
+	cstring = g_strdup_printf("#%2.2x%2.2x%2.2x",
+			(color->red >> 8) & 0xff,
+			(color->green >> 8) & 0xff,
+			(color->blue >> 8) & 0xff );
+	gkrellm_dup_string(color_string, cstring);
+	g_free(cstring);
+	}
+
+static void
+format_alt_color(gchar *src_string, gchar *buf, gint size, gchar *alt_color)
+	{
+	gint		len;
+	gchar		*s;
+
+	if (!buf || size < 1)
+		return;
+	--size;
+	*buf = '\0';
+	if (!src_string)
+		return;
+
+	for (s = src_string; *s != '\0' && size > 0; ++s)
+		{
+		len = 1;
+		if (*s == '$' && *(s + 1) != '\0')
+			{
+			switch(*(s + 1))
+				{
+				case 'A':
+					len = snprintf(buf, size, "%s", alt_color);
+					break;
+				default:
+					*buf = *s;
+					if (size > 1)
+						{
+						*(buf + 1) = *(s + 1);
+						++len;
+						}
+					break;
+				}
+			++s;
+			}
+		else
+			*buf = *s;
+		size -= len;
+		buf += len;
+		}
+	*buf = '\0';
+	}
+
 static gchar *
-utf8_from_strftime(gchar *buf)
+strftime_format(gchar *format, gchar *alt_color)
 	{
-	gchar	*utf8 = NULL;
-
-	if (   g_utf8_validate(buf, -1, NULL)
-		|| (utf8 = g_locale_to_utf8(buf, -1, NULL, NULL, NULL)) == NULL
-	   )
-		utf8 = g_strdup(buf);
-
-	return utf8;
-	}
-
-
-static void
-draw_cal(void)
-	{
-	struct tm 	*t;
-	gchar		*utf8, buf[32];
-	gint		w;
-
-	if (!cal_enable)
-		return;
-	if (_GK.client_mode)
-		t = gkrellm_client_server_time();
-	else
-		t = &gkrellm_current_tm;
-
-	strftime(buf, sizeof(buf), "%a", t);  /* Abbreviated weekday name */
-	utf8 = utf8_from_strftime(buf);
-	w = gkrellm_gdk_string_width(d_wday->text_style.font, utf8);
-	d_wday->x_off = (w < d_wday->w) ? (d_wday->w - w) / 2 : 0;
-	gkrellm_draw_decal_text(pcal, d_wday, utf8, -1);
-	g_free(utf8);
-
-	/*strftime(buf, sizeof(buf), "%e", t);*/  /* Day in month 1-31 */
-	snprintf(buf, sizeof(buf), "%d", t->tm_mday);
-	w = gkrellm_gdk_string_width(d_mday->text_style.font, buf);
-	d_mday->x_off = (w < d_mday->w) ? (d_mday->w - w) / 2 : 0;
-	gkrellm_draw_decal_text(pcal, d_mday, buf, -1);
-
-	strftime(buf, sizeof(buf), "%b", t); /* Abbreviated month name */
-	utf8 = utf8_from_strftime(buf);
-	gkrellm_draw_decal_text(pcal, d_month, utf8, -1);
-	g_free(utf8);
-
-	gkrellm_draw_panel_layers(pcal);
-	}
-
-
-static void
-draw_clock(gboolean update_minutes)
-	{
-	struct tm 	*t;
-	gchar		buf[32], sec_ampm[32], *utf8;
-	gint		w;
-
-	if (!clock_enable)
-		return;
+	struct tm	*t;
+	gchar		*locale_format = NULL;
+	gchar		buf1[512], buf2[512];
 
 	if (_GK.client_mode)
 		t = gkrellm_client_server_time();
 	else
 		t = &gkrellm_current_tm;
 
-	if (update_minutes)
-		{
-		if (clock_24hr_mode)
-			strftime(buf, sizeof(buf), STRFTIME_24_HOUR, t);
-		else
-			strftime(buf, sizeof(buf), STRFTIME_12_HOUR, t);
-		w = gkrellm_gdk_string_width(d_clock->text_style.font, buf);
-		d_clock->x_off = (w < d_clock->w) ? (d_clock->w - w) / 2 : 0;
-		gkrellm_draw_decal_text(pclock, d_clock, buf, -1);
-		}
-	if (   (!clock_24hr_mode && clock_12hr_sec_enable)
-		|| (clock_24hr_mode && clock_24hr_sec_enable)
-	   )
-		{
-		strftime(sec_ampm, sizeof(sec_ampm), "%S", t);	/* seconds 00-59 */
-		gkrellm_draw_decal_text(pclock, d_seconds, sec_ampm, -1);
-		}
-	else if (! clock_24hr_mode)
-		{
-		strftime(sec_ampm, sizeof(sec_ampm), "%p", t);	/* AM/PM */
-		if (sec_ampm[0] == '\0')		/* locale doesn't support AM/PM */
-			strcpy(sec_ampm, "--");
-		if (   g_utf8_validate(sec_ampm, -1, NULL)
-			|| (utf8 = g_locale_to_utf8(sec_ampm, -1, NULL, NULL,NULL)) == NULL
-		   )
-			utf8 = g_strdup(sec_ampm);
-		gkrellm_draw_decal_text(pclock, d_seconds, utf8, -1);
-		g_free(utf8);
-		}
-	gkrellm_draw_panel_layers(pclock);
-	}
+	locale_format = g_locale_from_utf8 (format, -1, NULL, NULL, NULL);
+	strftime(buf1, sizeof(buf1), locale_format, t);
+	g_free (locale_format);
+	format_alt_color(buf1, buf2, sizeof(buf2), alt_color);
 
-static void
-update_clock(void)
-	{
-	struct tm		*ptm;
-	gboolean		update_minutes = FALSE;
-	static gint		min_prev, hour_prev = -1, sec_prev = -1;
-	ChimeData		*chime;
+// printf("%s\n", buf2);
 
-	if (_GK.client_mode)
-		{
-		ptm = gkrellm_client_server_time();
-		if (sec_prev == ptm->tm_sec)
-			return;
-		sec_prev = ptm->tm_sec;
-		}
-	else
-		{
-		if (!GK.second_tick)
-			return;
-		ptm = &gkrellm_current_tm;
-		}
-
-	if (   ptm->tm_min != min_prev
-		|| ptm->tm_hour != hour_prev
-	   )
-		{
-		update_minutes = TRUE;
-		draw_cal();
-		if (ptm->tm_hour != hour_prev && hour_prev != -1)
-			{
-			if (!chime_block && hour_chime_command && *hour_chime_command) 
-				{
-				chime = g_new0(ChimeData, 1);
-				chime -> command = strdup(hour_chime_command);
-				chime -> count = loop_chime_enable ? ptm->tm_hour : 1;
-				g_thread_create(chime_func, chime, FALSE, NULL);
-				}
-			}
-		else
-			{
-			if (   !chime_block && (ptm->tm_min % 15) == 0
-				&& quarter_chime_command && *quarter_chime_command
-			   ) 
-				{
-				chime = g_new0(ChimeData, 1);
-				chime -> command = strdup(quarter_chime_command);
-				chime -> count = 1;
-				g_thread_create(chime_func, chime, FALSE, NULL);
-				}
-			}
-		}
-	draw_clock(update_minutes);
-	min_prev = ptm->tm_min;
-	hour_prev = ptm->tm_hour;
+	return g_locale_to_utf8 (buf2, -1, NULL, NULL, NULL);
 	}
 
 static gint
@@ -322,89 +254,24 @@ cal_visibility(void)
 	}
 
 static void
-create_calendar(GtkWidget *vbox, gint first_create)
+create_calendar_panel(GtkWidget *vbox, gint first_create)
 	{
-	GkrellmStyle	*style;
-	Extents			ext;
-	gchar			buf[32], *utf8, *str = NULL;
-	gint			m, d, w;
-	struct tm		tm = *gkrellm_get_current_time();
+	GkrellmStyle		*style;
+	GkrellmTextstyle	*ts;
+	gchar				*cal_string;
 
 	if (first_create)
 		pcal = gkrellm_panel_new0();
 	style = gkrellm_meter_style(cal_style_id);
-	ext.ts = *gkrellm_meter_textstyle(cal_style_id);
 
-	/* Get max weekday name height for current locale
-	*/
-	wday_extents.width = wday_extents.height = 0;
-	wday_extents.baseline = wday_extents.y_ink = 0;
-	for (d = 0; d < 6; ++d)
-		{
-		tm.tm_wday = d;
-		strftime(buf, sizeof(buf), "%a", &tm);  /* Abbreviated weekday name */
-		utf8 = utf8_from_strftime(buf);
-		string_extents(utf8, &ext);
-		wday_extents.baseline = ext.baseline;	/* not used */
-		if (ext.width > wday_extents.width)
-			wday_extents.width = ext.width;
-		if (ext.height > wday_extents.height)
-			{
-			str = utf8;
-			wday_extents.height = ext.height;	/* not used */
-			wday_extents.y_ink = ext.y_ink;		/* not used */
-			}
-		else
-			g_free(utf8);
-		}
-	wday_extents.ts = *gkrellm_meter_textstyle(cal_style_id);
-	d_wday = gkrellm_create_decal_text(pcal, str, &wday_extents.ts,
-				style, 0, -1 /*top margin*/, wday_extents.width + 2);
-	g_free(str);
+	ts = gkrellm_meter_alt_textstyle(cal_style_id);
+	get_color_name(&ts->color, &cal_alt_color_string);
 
-	mday_extents.ts = *gkrellm_meter_alt_textstyle(cal_style_id);
-	string_extents("28", &mday_extents);
-	d_mday = gkrellm_create_decal_text(pcal, "28", &mday_extents.ts,
-				style, 0, -1 /*top margin*/, mday_extents.width + 2);
+	cal_string = strftime_format(cal_format, cal_alt_color_string);
+	d_cal = gkrellm_create_decal_text_markup(pcal, cal_string,
+				gkrellm_meter_textstyle(cal_style_id), style, -1, -1, -1);
+	g_free(cal_string);
 
-	/* Get max month name height for current locale
-	*/
-	month_extents.width = month_extents.height = 0;
-	month_extents.baseline = month_extents.y_ink = 0;
-	for (m = 0; m < 11; ++m)
-		{
-		tm.tm_mon = m;
-		strftime(buf, sizeof(buf), "%b", &tm); /* Abbreviated month name */
-		utf8 = utf8_from_strftime(buf);
-		string_extents(utf8, &ext);
-		month_extents.baseline = ext.baseline;	/* not used */
-		if (ext.width > month_extents.width)
-			month_extents.width = ext.width;
-		if (ext.height > month_extents.height)
-			{
-			str = utf8;
-			month_extents.height = ext.height;	/* not used */
-			month_extents.y_ink = ext.y_ink;		/* not used */
-			}
-		else
-			g_free(utf8);
-		}
-	month_extents.ts = *gkrellm_meter_textstyle(cal_style_id);
-	d_month = gkrellm_create_decal_text(pcal, str, &month_extents.ts,
-				style, 0, -1 /*top margin*/, month_extents.width + 2);
-	g_free(str);
-
-	w = d_wday->w + d_mday->w + d_month->w;
-	d_wday->x = (gkrellm_chart_width() - w) / 2;
-	d_mday->x = d_wday->x + d_wday->w;
-	d_month->x = d_mday->x + d_mday->w;
-	if (d_wday->h < d_mday->h)
-		{
-		d_wday->y = d_mday->y + d_mday->h - d_wday->h - 1;
-		d_month->y = d_wday->y;
-		}
-	else if (d_mday->h < d_wday->h)
-		d_mday->y = d_wday->y + d_wday->h - d_mday->h - 1;
 	gkrellm_panel_configure(pcal, NULL, style);
 	gkrellm_panel_create(vbox, mon_cal, pcal);
 
@@ -426,24 +293,55 @@ create_calendar(GtkWidget *vbox, gint first_create)
 	gkrellm_setup_launcher(pcal, &cal_launch, METER_PANEL_TYPE, 0);
 
 	cal_visibility();
-	draw_cal();
 	}
 
 static void
-set_seconds_display()
+draw_cal(void)
 	{
-	if (   !clock_24hr_mode
-		|| (clock_24hr_mode && clock_24hr_sec_enable)
-	   )
+	gchar			*utf8, *cal_string;
+	gint			w, h;
+
+	if (!cal_enable)
+		return;
+
+	cal_string = strftime_format(cal_format, cal_alt_color_string);
+	if (!g_utf8_validate(cal_string, -1, NULL))
 		{
-		gkrellm_move_decal(pclock, d_clock, x_clock_seconds, d_clock->y);
-		gkrellm_make_decal_visible(pclock, d_seconds);
+	    if ((utf8 = g_locale_to_utf8(cal_string, -1, NULL, NULL, NULL))
+				!= NULL)
+			{
+			g_free(cal_string);
+			cal_string = utf8;
+			}
 		}
-	else
+
+	/* Get string extents in case a string change => need to resize the
+	|  panel.  A string change can also change the decal y_ink (possibly
+	|  without changing decal size).
+	*/
+	gkrellm_text_markup_extents(d_cal->text_style.font,
+			cal_string, strlen(cal_string), &w, &h, NULL, &d_cal->y_ink);
+
+	w += d_cal->text_style.effect;
+	if (h + d_cal->text_style.effect != d_cal->h)
 		{
-		gkrellm_move_decal(pclock, d_clock, x_clock_no_seconds, d_clock->y);
-		gkrellm_make_decal_invisible(pclock, d_seconds);
+		gkrellm_panel_destroy(pcal);
+		create_calendar_panel(cal_vbox, TRUE);
 		}
+
+	gkrellm_draw_decal_markup(pcal, d_cal, cal_string);
+	gkrellm_decal_text_set_offset(d_cal, (d_cal->w - w) / 2, 0);
+
+	gkrellm_draw_panel_layers(pcal);
+	g_free(cal_string);
+	}
+
+static void
+create_calendar(GtkWidget *vbox, gint first_create)
+	{
+	cal_vbox = vbox;
+	create_calendar_panel(vbox, first_create);
+	draw_cal();
 	}
 
 static void
@@ -462,30 +360,23 @@ clock_visibility(void)
 	}
 
 static void
-create_clock(GtkWidget *vbox, gint first_create)
+create_clock_panel(GtkWidget *vbox, gint first_create)
 	{
-	GkrellmStyle	*style;
+	GkrellmStyle		*style;
+	GkrellmTextstyle	*ts;
+	gchar				*clock_string;
 
 	if (first_create)
 		pclock = gkrellm_panel_new0();
 	style = gkrellm_meter_style(clock_style_id);
 
-	time_extents.ts = *gkrellm_meter_textstyle(clock_style_id);
-	string_extents("00:00", &time_extents);
-	sec_extents.ts = *gkrellm_meter_alt_textstyle(clock_style_id);
-	string_extents("8M", &sec_extents);
-	ampm_extents.ts = *gkrellm_meter_alt_textstyle(clock_style_id);
-	string_extents("8M", &ampm_extents);
+	ts = gkrellm_meter_alt_textstyle(clock_style_id);
+	get_color_name(&ts->color, &clock_alt_color_string);
 
-	d_clock = gkrellm_create_decal_text(pclock, "88", &time_extents.ts,
-				style, 0, -1 /*top margin*/, time_extents.width + 2);
-	d_seconds = gkrellm_create_decal_text(pclock, "88", &sec_extents.ts,
-				style, 0, -1, sec_extents.width + 2);
-
-	x_clock_seconds = (gkrellm_chart_width() - d_clock->w - d_seconds->w) / 2;
-	x_clock_no_seconds = (gkrellm_chart_width() - d_clock->w) / 2;
-	d_seconds->x = x_clock_seconds + d_clock->w + 2;
-	d_seconds->y = d_clock->y + d_clock->h - d_seconds->h;
+	clock_string = strftime_format(clock_format, clock_alt_color_string);
+	d_clock = gkrellm_create_decal_text_markup(pclock, clock_string,
+				gkrellm_meter_textstyle(clock_style_id), style, -1, -1, -1);
+	g_free(clock_string);
 
 	gkrellm_panel_configure(pclock, NULL, style);
 	gkrellm_panel_create(vbox, mon_clock, pclock);
@@ -499,12 +390,117 @@ create_clock(GtkWidget *vbox, gint first_create)
 		}
 	gkrellm_setup_launcher(pclock, &clock_launch, METER_PANEL_TYPE, 0);
 
-	set_seconds_display();
-
 	clock_visibility();
+	}
+
+static void
+draw_clock(gboolean check_size)
+	{
+	gchar	*utf8, *clock_string;
+	gint	w,h;
+
+	if (!clock_enable)
+		return;
+
+	clock_string = strftime_format(clock_format, clock_alt_color_string);
+	if (!g_utf8_validate(clock_string, -1, NULL))
+		{
+		if ((utf8 = g_locale_to_utf8(clock_string, -1, NULL, NULL, NULL))
+				!= NULL)
+			{
+			g_free(clock_string);
+			clock_string = utf8;
+			}
+		}
+
+	if (check_size)
+		{
+		/* Get string extents in case a string change => need to resize the
+		|  panel.  A string change can also change the decal y_ink (possibly
+		|  without changing decal size).
+		*/
+		gkrellm_text_markup_extents(d_clock->text_style.font,
+				clock_string, strlen(clock_string),
+				&w, &h, NULL, &d_clock->y_ink);
+		w += d_clock->text_style.effect;
+		if (h + d_clock->text_style.effect != d_clock->h)
+			{
+			gkrellm_panel_destroy(pclock);
+			create_clock_panel(clock_vbox, TRUE);
+			}
+		gkrellm_decal_text_set_offset(d_clock, (d_clock->w - w) / 2, 0);
+		}
+	gkrellm_draw_decal_markup(pclock, d_clock, clock_string);
+	gkrellm_draw_panel_layers(pclock);
+	g_free(clock_string);
+	}
+
+
+static void
+create_clock(GtkWidget *vbox, gint first_create)
+	{
+	clock_vbox = vbox;
+	create_clock_panel(vbox, first_create);
 	draw_clock(TRUE);
 	}
 
+static void
+update_clock(void)
+	{
+	struct tm		*ptm;
+	static gint		min_prev, hour_prev = -1, sec_prev = -1;
+	ChimeData		*chime;
+	gboolean		hour_tick;
+
+	if (_GK.client_mode)
+		{
+		ptm = gkrellm_client_server_time();
+		if (sec_prev == ptm->tm_sec)
+			return;
+		sec_prev = ptm->tm_sec;
+		}
+	else
+		{
+		if (!GK.second_tick)
+			return;
+		ptm = &gkrellm_current_tm;
+		}
+
+	hour_tick = (ptm->tm_hour != hour_prev);
+	if (   ptm->tm_min != min_prev
+		|| hour_tick
+	   )
+		{
+		draw_cal();
+		if (ptm->tm_hour != hour_prev && hour_prev != -1)
+			{
+			if (!chime_block && hour_chime_command && *hour_chime_command)
+				{
+				chime = g_new0(ChimeData, 1);
+				chime -> command = strdup(hour_chime_command);
+				chime -> count = loop_chime_enable ? ptm->tm_hour : 1;
+				g_thread_create(chime_func, chime, FALSE, NULL);
+				}
+			}
+		else
+			{
+			if (   !chime_block && (ptm->tm_min % 15) == 0
+				&& quarter_chime_command && *quarter_chime_command
+			   )
+				{
+				chime = g_new0(ChimeData, 1);
+				chime -> command = strdup(quarter_chime_command);
+				chime -> count = 1;
+				g_thread_create(chime_func, chime, FALSE, NULL);
+				}
+			}
+		}
+	draw_clock(hour_tick);
+	min_prev = ptm->tm_min;
+	hour_prev = ptm->tm_hour;
+	}
+
+
 #define	CLOCK_CONFIG_KEYWORD	"clock_cal"
 
 static void
@@ -524,11 +520,14 @@ save_clock_cal_config(FILE *f)
 				quarter_chime_command);
     fprintf(f, "%s loop_chime_enable %d\n", CLOCK_CONFIG_KEYWORD,
                loop_chime_enable);
-	fprintf(f, "%s clock_options %d %d %d %d\n", CLOCK_CONFIG_KEYWORD,
-				clock_enable, clock_12hr_sec_enable, clock_24hr_mode, 
-				clock_24hr_sec_enable);
+	fprintf(f, "%s clock_options %d\n", CLOCK_CONFIG_KEYWORD,
+				clock_enable);
 	fprintf(f, "%s cal_options %d\n", CLOCK_CONFIG_KEYWORD,
 				cal_enable);
+	fprintf(f, "%s cal_format %s\n", CLOCK_CONFIG_KEYWORD,
+				cal_format);
+	fprintf(f, "%s clock_format %s\n", CLOCK_CONFIG_KEYWORD,
+				clock_format);
 	}
 
 static void
@@ -555,10 +554,13 @@ load_clock_cal_config(gchar *arg)
 		else if (!strcmp(config, "loop_chime_enable"))
 			sscanf(item, "%d", &loop_chime_enable);
 		else if (!strcmp(config, "clock_options"))
-			sscanf(item, "%d %d %d %d", &clock_enable, &clock_12hr_sec_enable,
-					&clock_24hr_mode, &clock_24hr_sec_enable);
+			sscanf(item, "%d", &clock_enable);
 		else if (!strcmp(config, "cal_options"))
 			sscanf(item, "%d", &cal_enable);
+		else if (!strcmp(config, "cal_format"))
+			gkrellm_dup_string(&cal_format, item);
+		else if (!strcmp(config, "clock_format"))
+			gkrellm_dup_string(&clock_format, item);
 		}
 	}
 
@@ -571,36 +573,16 @@ static GtkWidget	*cal_launch_entry,
                     *quarter_chime_entry,
 					*clock_enable_button,
 					*cal_enable_button,
-					*loop_chime_button,
-					*clock_24hr_button,
-					*clock_24hr_sec_button,
-					*clock_12hr_sec_button;
+					*loop_chime_button;
 
-
-static void
-set_seconds_buttons_sensitivity(void)
-	{
-	if (clock_24hr_mode)
-		{
-		gtk_widget_set_sensitive(clock_24hr_sec_button, TRUE);
-		gtk_widget_set_sensitive(clock_12hr_sec_button, FALSE);
-		}
-	else
-		{
-		gtk_widget_set_sensitive(clock_24hr_sec_button, FALSE);
-		gtk_widget_set_sensitive(clock_12hr_sec_button, TRUE);
-		}
-	}
+static GtkWidget	*cal_format_combo_box,
+					*clock_format_combo_box;
 
 
 static void
 cb_clock_cal(GtkWidget *widget, gpointer data)
 	{
 	loop_chime_enable = GTK_TOGGLE_BUTTON(loop_chime_button)->active;
-	clock_24hr_mode = GTK_TOGGLE_BUTTON (clock_24hr_button)->active;
-	clock_12hr_sec_enable = GTK_TOGGLE_BUTTON (clock_12hr_sec_button)->active;
-	clock_24hr_sec_enable = GTK_TOGGLE_BUTTON (clock_24hr_sec_button)->active;
-	set_seconds_buttons_sensitivity();
 
 	clock_enable = GTK_TOGGLE_BUTTON(clock_enable_button)->active;
 	clock_visibility();
@@ -608,7 +590,6 @@ cb_clock_cal(GtkWidget *widget, gpointer data)
 	cal_enable = GTK_TOGGLE_BUTTON(cal_enable_button)->active;
 	cal_visibility();
 
-	set_seconds_display();
 	draw_cal();
 	draw_clock(TRUE);
 	}
@@ -644,17 +625,85 @@ cb_chime_entry(GtkWidget *widget, gpointer data)
 				gkrellm_gtk_entry_get_text(&quarter_chime_entry));
 	}
 
+static void
+cal_format_cb(GtkWidget *widget, gpointer data)
+	{
+	gchar	*s, *check;
+	GtkWidget *entry;
+
+	entry = gtk_bin_get_child(GTK_BIN(cal_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
+
+	check = strftime_format(s, cal_alt_color_string);
+
+	/* In case Pango markup tags, don't accept line unless valid markup.
+	|  Ie, markup like <span ...> xxx </span> or <b> xxx </b>
+	*/
+	if (   strchr(check, '<') != NULL
+	    && !pango_parse_markup(check, -1, 0, NULL, NULL, NULL, NULL)
+	   )
+		{
+		g_free(check);
+		return;
+		}
+
+	g_free(check);
+	if (gkrellm_dup_string(&cal_format, s))
+		draw_cal();
+	}
+
+static void
+clock_format_cb(GtkWidget *widget, gpointer data)
+	{
+	gchar	*s, *check;
+	GtkWidget *entry;
+
+	entry = gtk_bin_get_child(GTK_BIN(clock_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
+
+	check = strftime_format(s, clock_alt_color_string);
+
+	if (   strchr(check, '<') != NULL
+	    && !pango_parse_markup(check, -1, 0, NULL, NULL, NULL, NULL)
+	   )
+		{
+		g_free(check);
+		return;
+		}
+
+	g_free(check);
+	if (gkrellm_dup_string(&clock_format, s))
+		draw_clock(TRUE);
+	}
+
 static void
 config_destroyed(void)
 	{
 	chime_block = FALSE;
 	}
 
+
+static gchar    *clock_info_text[] =
+{
+N_("<h>Clock/Calendar Format Strings\n"),
+N_("The display format strings should contain strftime conversion\n"
+"characters and Pango text attribute markup strings.\n"),
+"\n",
+N_("For the clock, the provided default strings will display a 12 hour\n"
+"clock with seconds, a 12 hour clock with AM/PM indicator, or a 24 hour\n"
+"clock with seconds.\n"),
+"\n",
+N_("The special $A substitution variable expands to the current theme\n"
+"alternate color and is for use with the Pango \"foreground\" attribute.\n")
+};
+
+
 static void
 create_clock_tab(GtkWidget *tab_vbox)
 	{
 	GtkWidget	*tabs;
-	GtkWidget	*table, *vbox, *vbox1, *hbox;
+	GtkWidget	*table, *vbox, *vbox1, *hbox, *label, *text;
+	gint		i;
 
 	tabs = gtk_notebook_new();
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
@@ -664,45 +713,49 @@ create_clock_tab(GtkWidget *tab_vbox)
 
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Options"));
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Calendar"), 4, FALSE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox, _("Calendar"), 4, 0, TRUE);
 	gkrellm_gtk_check_button_connected(vbox1, &cal_enable_button,
 				cal_enable, FALSE, FALSE, 2,
 				cb_clock_cal, NULL,
 				_("Enable"));
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 6);
+	label = gtk_label_new(_("Display format string:"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+	cal_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(vbox1), cal_format_combo_box, TRUE, TRUE, 0);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(cal_format_combo_box), cal_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(cal_format_combo_box), DEFAULT_CAL_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(cal_format_combo_box), ALT1_CAL_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(cal_format_combo_box), ALT2_CAL_FORMAT);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(cal_format_combo_box), 0);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(cal_format_combo_box)), "changed",
+			G_CALLBACK(cal_format_cb), NULL);
 
-	vbox1 = gkrellm_gtk_framed_vbox_end(vbox, _("Clock"), 4, FALSE, 0, 2);
-
+	vbox1 = gkrellm_gtk_category_vbox(vbox, _("Clock"), 4, 0, TRUE);
 	gkrellm_gtk_check_button_connected(vbox1, &clock_enable_button,
 				clock_enable, FALSE, FALSE, 6,
 				cb_clock_cal, NULL,
 				_("Enable"));
-
-	gkrellm_gtk_check_button_connected(vbox1, &clock_24hr_button,
-				clock_24hr_mode, FALSE, FALSE, 0,
-				cb_clock_cal, NULL, 
-				_("Display 24 hour instead of 12 hour time"));
-
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
-	gkrellm_gtk_check_button_connected(hbox, &clock_12hr_sec_button,
-			clock_12hr_sec_enable, FALSE, FALSE, 20,
-			cb_clock_cal, NULL,
-			_("Show seconds instead of am/pm in 12 hour time mode"));
-
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
-	gkrellm_gtk_check_button_connected(hbox, &clock_24hr_sec_button,
-			clock_24hr_sec_enable, FALSE, FALSE, 20,
-			cb_clock_cal, NULL,
-			_("Show seconds in 24 hour time mode"));
-
-	set_seconds_buttons_sensitivity();
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 6);
+	label = gtk_label_new(_("Display format string:"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+	clock_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(vbox1), clock_format_combo_box, TRUE, TRUE, 0);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(clock_format_combo_box), clock_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(clock_format_combo_box), DEFAULT_CLOCK_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(clock_format_combo_box), ALT1_CLOCK_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(clock_format_combo_box), ALT2_CLOCK_FORMAT);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(clock_format_combo_box), 0);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(clock_format_combo_box)), "changed",
+			G_CALLBACK(clock_format_cb), NULL);
 
 /* -- Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Clock Chime Commands"),
-				4, FALSE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox, _("Clock Chime Commands"),
+				4, 0, TRUE);
 	table = gkrellm_gtk_launcher_table_new(vbox1, 2);
 
 	gkrellm_gtk_config_launcher(table, 0, &hour_chime_entry, NULL,
@@ -721,22 +774,29 @@ create_clock_tab(GtkWidget *tab_vbox)
 	g_signal_connect(G_OBJECT(quarter_chime_entry), "activate",
 			G_CALLBACK(cb_chime_entry), GINT_TO_POINTER(0x10));
 
-	gkrellm_gtk_check_button_connected(vbox, &loop_chime_button,
+	gkrellm_gtk_check_button_connected(vbox1, &loop_chime_button,
 			loop_chime_enable, FALSE, FALSE, 6,
 			cb_clock_cal, NULL,
 			_("Loop hour chime command"));
 
-	vbox = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
-			4, FALSE, 0, 2);
+	vbox = gkrellm_gtk_category_vbox(vbox, _("Launch Commands"),
+				4, 0, TRUE);
 	table = gkrellm_gtk_launcher_table_new(vbox, 2);
 
 	gkrellm_gtk_config_launcher(table, 0,  &cal_launch_entry,
 				&cal_tooltip_entry, _("Calendar"), &cal_launch);
+
+	/* --Info tab */
+	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Info"));
+	text = gkrellm_gtk_scrolled_text_view(vbox, NULL,
+				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	for (i = 0; i < sizeof(clock_info_text)/sizeof(gchar *); ++i)
+		gkrellm_gtk_text_view_append(text, _(clock_info_text[i]));
 	g_signal_connect(G_OBJECT(cal_launch_entry), "changed",
 			G_CALLBACK(cb_launch_entry), GINT_TO_POINTER(1));
 	g_signal_connect(G_OBJECT(cal_tooltip_entry), "changed",
 			G_CALLBACK(cb_launch_entry), GINT_TO_POINTER(1));
-	
+
 	gkrellm_gtk_config_launcher(table, 1,  &clock_launch_entry,
 				&clock_tooltip_entry, _("Clock"), &clock_launch);
 	g_signal_connect(G_OBJECT(clock_launch_entry), "changed",
@@ -774,12 +834,12 @@ gkrellm_init_clock_monitor(void)
 	{
 	clock_enable = TRUE;
     monitor_clock.name = _(monitor_clock.name);
-	clock_12hr_sec_enable = TRUE;
-	clock_24hr_sec_enable = TRUE;
     loop_chime_enable   = FALSE;
     hour_chime_command = g_strdup("");
     quarter_chime_command = g_strdup("");
 
+	gkrellm_dup_string(&clock_format, DEFAULT_CLOCK_FORMAT);
+
 	clock_style_id = gkrellm_add_meter_style(&monitor_clock, CLOCK_STYLE_NAME);
 	mon_clock = &monitor_clock;
 	return &monitor_clock;
@@ -814,6 +874,8 @@ gkrellm_init_cal_monitor(void)
 	cal_enable = TRUE;
     monitor_cal.name = _(monitor_cal.name);
 
+	gkrellm_dup_string(&cal_format, DEFAULT_CAL_FORMAT);
+
 	cal_style_id = gkrellm_add_meter_style(&monitor_cal, CAL_STYLE_NAME);
 	mon_cal = &monitor_cal;
 	return &monitor_cal;
