@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2008 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,6 +17,18 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -257,7 +269,7 @@ assign_textcolor(GkrellmStyle *style, gchar *arg, gint AorB)
 	effect = gkrellm_cut_quoted_string(s, &s);
 	if (*color == '\0' || *shadowcolor == '\0' || *effect == '\0')
 		{
-		printf(_("Bad textcolor line %s\n"), arg);
+		g_warning(_("Bad textcolor line %s\n"), arg);
 		g_free(values);
 		return;
 		}
@@ -499,7 +511,7 @@ assign_style(gchar *debug_name, GList *style_list, gint index,
 	if (index++ == 0)		/* style == style_list */
 		{
 		if (override)
-			printf("Bad override on DEFAULT: %s %s %d\n",
+			g_warning("Bad override on DEFAULT: %s %s %d\n",
 					debug_name, arg, entry_flag);
 		for (list = style_list->next; list; list = list->next, ++index)
 			{
@@ -632,7 +644,7 @@ assign_gkrellmrc_style(gchar *source_line, gchar *area, gchar *string)
 		}
 	if (!mon_name || !entry || !*entry || !arg)
 		{
-		printf("StyleXXX ?: %s\n", source_line);
+		g_warning("StyleXXX ?: %s\n", source_line);
 		g_free(custom_name);
 		return;
 		}
@@ -655,7 +667,7 @@ assign_gkrellmrc_style(gchar *source_line, gchar *area, gchar *string)
 		}
 	else
 		{
-		printf("StyleXXX ?: %s\n", source_line);
+		g_warning("StyleXXX ?: %s\n", source_line);
 		g_free(custom_name);
 		return;
 		}
@@ -1037,13 +1049,13 @@ gkrellm_load_piximage_from_inline(gchar *name, const guint8 *data,
 		name = fname;
 		im = gkrellm_piximage_new_from_file(fname);
 		if (im == NULL)
-			printf(_("  Cannot load file image: %s\n"), fname);
+			g_warning(_("  Cannot load file image: %s\n"), fname);
 		}
 	if (im == NULL && data)
 		{
 		im = gkrellm_piximage_new_from_inline(data, copy_pixels);
 		if (im == NULL)
-			printf(_("  Cannot load GdkPixbuf inline data.\n"));
+			g_warning(_("  Cannot load GdkPixbuf inline data.\n"));
 		}
 	if (im && image)
 		{
@@ -1068,13 +1080,13 @@ gkrellm_load_piximage(gchar *name, gchar **xpm, GkrellmPiximage **image,
 		name = fname;
 		im = gkrellm_piximage_new_from_file(fname);
 		if (im == NULL)
-			printf(_("  Cannot load file image: %s\n"), fname);
+			g_warning(_("  Cannot load file image: %s\n"), fname);
 		}
 	if (im == NULL && xpm)
 		{
 		im = gkrellm_piximage_new_from_xpm_data(xpm);
 		if (im == NULL)
-			printf(_("  Cannot load xpm: %s\n"), name);
+			g_warning(_("  Cannot load xpm: %s\n"), name);
 		}
 	if (im && image)
 		{
@@ -1100,7 +1112,7 @@ load_from_piximage_list(gchar *name, GList *image_list,
 		list->data = (gpointer) im;
 		}
 	else
-		printf("Bad index %d for image list (meter/panel problem?)\n", index);
+		g_warning("Bad index %d for image list (meter/panel problem?)\n", index);
 	}
 
 static void
@@ -1738,7 +1750,7 @@ parse_config_line(gchar *line, struct _config *cf, gint size)
 			++s;
 		if (!*s || *s == '\n')
 			{
-			printf(_("Incomplete config line:\n    %s\n"), line);
+			g_warning(_("Incomplete config line:\n    %s\n"), line);
 			continue;
 			}
 		if (conf->value)
@@ -2237,9 +2249,10 @@ gkrellm_theme_config(void)
 	/* Warn theme developers!
 	*/
 	if (!_GK.use_top_bottom_margins && _GK.command_line_theme)
-		fprintf(stderr,
-				"Warning: Top and bottom meter/panel margins are not set.\n"
-				"         Do not depend on borders!!\n");
+		{
+		g_warning("Top and bottom meter/panel margins are not set.\n" \
+				" Do not depend on borders!\n");
+		}
 	}
 
 
@@ -2290,7 +2303,7 @@ gkrellm_load_user_config(GkrellmMonitor *mon_only, gboolean monitor_values)
 		_GK.enable_hostname = TRUE;
 		_GK.hostname_short = FALSE;
 		_GK.enable_system_name = FALSE;
-		_GK.chart_width = 60;
+		_GK.chart_width = 100;
 		_GK.update_HZ = 10;
 		_GK.theme_scale = 100;
 		_GK.float_factor = 1.0;
@@ -2422,7 +2435,7 @@ gkrellm_save_user_config(void)
 	f = fopen(config_new, "w");
 	if (f == NULL)
 		{
-		printf(_("Cannot open config file %s for writing.\n"), config_new);
+		g_warning(_("Cannot open config file %s for writing.\n"), config_new);
 		g_free(config_new);
 		g_free(config);
 		return;
@@ -2501,7 +2514,19 @@ gkrellm_save_user_config(void)
 		fclose(ff);
 		}
 	fclose(f);
-	rename(config_new, config);
+#if defined(WIN32)
+	/* windows rename() does not allow overwriting existing files! */
+	unlink(config);
+#endif
+	i = rename(config_new, config);
+	if (i != 0)
+		{
+		g_warning("Cannot rename new config file %s to %s.\n", config_new, config);
+		g_free(config);
+		g_free(config_new);
+		// NOTE: _GK.config_modified will stay true because saving failed
+		return;
+		}
 
 #if defined (S_IRUSR)
 	mode = (S_IRUSR | S_IWUSR);
