@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2008 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,6 +17,18 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -262,7 +274,7 @@ gkrellm_disk_units_are_blocks(void)
 	{
 	units_are_blocks = TRUE;
 	}
-	
+
 void
 gkrellm_disk_assign_data_by_device(gint device_number, gint unit_number,
 			guint64 rb, guint64 wb, gboolean virtual)
@@ -418,7 +430,7 @@ format_disk_data(DiskMon *disk, gchar *src_string, gchar *buf, gint size)
 		size -= len;
 		buf += len;
 		}
-	*buf = '\0';	
+	*buf = '\0';
 	}
 
 static void
@@ -912,7 +924,7 @@ save_disk_config(FILE *f)
 		|  a disk saved into the config here should be artificially created
 		|  should it not be present at next gkrellm startup.
 		*/
-		have_enabled_subdisks = (disk->subdisk == -1) ? 
+		have_enabled_subdisks = (disk->subdisk == -1) ?
 					any_enabled_subdisks(list->next) : FALSE;
 
 		fprintf(f, "%s device %s %d %d %d %d %d %d %d\n",
@@ -1036,7 +1048,7 @@ static GtkTreeRowReference	*row_reference;
 static GtkTreeSelection		*selection;
 
 static GtkWidget			*launch_vbox,
-							*text_format_combo;
+							*text_format_combo_box;
 
 static GtkWidget			*alert_button;
 
@@ -1273,7 +1285,7 @@ cb_enable(GtkCellRendererText *cell, gchar *path_string, gpointer data)
 				DISK_COLUMN, &disk,
 				-1);
 	gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
-				ENABLE_COLUMN, !enabled, 
+				ENABLE_COLUMN, !enabled,
 				-1);
 	if (enabled)
 		{
@@ -1303,8 +1315,10 @@ cb_text_format(GtkWidget *widget, gpointer data)
 	GList	*list;
 	DiskMon	*disk;
 	gchar	*s;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
+	entry = gtk_bin_get_child(GTK_BIN(text_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 	gkrellm_locale_dup_string(&text_format, s, &text_format_locale);
 	for (list = disk_mon_list; list; list = list->next)
 		{
@@ -1403,21 +1417,20 @@ create_disk_tab(GtkWidget *tab_vbox)
 	vbox1 = gkrellm_gtk_category_vbox(vbox,
 				_("Format String for Chart Labels"),
 				4, 0, TRUE);
-	text_format_combo = gtk_combo_new();
-	gtk_box_pack_start(GTK_BOX(vbox1), text_format_combo, FALSE, FALSE, 2);
-	list = NULL;
-	list = g_list_append(list, text_format);
-	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
-	list = g_list_append(list, "\\c\\f$M\\n$T");
-	list = g_list_append(list, "\\c\\f$M\\b$T");
-	list = g_list_append(list,
-					_("\\f\\ww\\r\\f$M\\D2\\f\\ar\\. $r\\D1\\f\\aw\\. $w"));
-	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(text_format_combo), TRUE);
-	g_list_free(list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
-			text_format);
-	g_signal_connect(G_OBJECT(GTK_COMBO(text_format_combo)->entry), "changed",
+	text_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(vbox1), text_format_combo_box, FALSE, FALSE, 0);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		text_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		DEFAULT_TEXT_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		"\\c\\f$M\\n$T");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		"\\c\\f$M\\b$T");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		_("\\f\\ww\\r\\f$M\\D2\\f\\ar\\. $r\\D1\\f\\aw\\. $w"));
+	gtk_combo_box_set_active(GTK_COMBO_BOX(text_format_combo_box), 0);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(text_format_combo_box)), "changed",
 			G_CALLBACK(cb_text_format), NULL);
 
 	vbox1 = gkrellm_gtk_category_vbox(vbox,
