@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2008 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,6 +17,18 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -24,7 +36,9 @@
 #include "gkrellm-sysdeps.h"
 
 
-#define	DEFAULT_DATA_FORMAT	("$t - $f free")
+#define	DEFAULT_DATA_FORMAT	(_("$t - $f free"))
+#define	ALT1_DATA_FORMAT (_("$t - $u used"))
+#define	ALT2_DATA_FORMAT (_("$t - $U"))
 
 
   /* Values for force_fs_check	*/
@@ -599,7 +613,7 @@ mount_command(FSmon *fs)
 	if (fs->is_mounted)
 		{
 		if (fs->fstab_mounting)
-			snprintf(cmd, sizeof(cmd), "umount %s", fs->mount.directory);
+			snprintf(cmd, sizeof(cmd), "umount '%s'", fs->mount.directory);
 		else
 			snprintf(cmd, sizeof(cmd), "%s", fs->launch_umount.command);
 		fs->label_is_data = FALSE;
@@ -613,7 +627,7 @@ mount_command(FSmon *fs)
 		if (fs->ejectable)
 			close_tray(fs);
 		if (fs->fstab_mounting)
-			snprintf(cmd, sizeof(cmd), "mount %s", fs->mount.directory);
+			snprintf(cmd, sizeof(cmd), "mount '%s'", fs->mount.directory);
 		else
 			snprintf(cmd, sizeof(cmd), "%s", fs->launch_mount.command);
 		fs->blocks = fs->bfree = fs->bavail = fs->bsize = 0;
@@ -1299,14 +1313,15 @@ fs_config_save(FILE *f)
 	{
 	GList	*list;
 	FSmon	*fs;
-	gchar	quoted_label[64];
+	gchar	quoted_label[64], quoted_dir[512];
 
 	for (list = fs_mon_list; list; list = list->next)
 		{
 		fs = (FSmon *) list->data;
 		snprintf(quoted_label, sizeof(quoted_label), "\"%s\"", fs->label);
+		snprintf(quoted_dir, sizeof(quoted_dir), "\"%s\"",fs->mount.directory);
 		fprintf(f, "%s %s %s %d %d %d %d %d\n", FS_CONFIG_KEYWORD,
-				quoted_label, fs->mount.directory,
+				quoted_label, quoted_dir,
 				fs->fstab_mounting, fs->secondary,
 				fs->show_if_mounted, fs->label_is_data, fs->ejectable);
 		if (*(fs->launch_mount.command))
@@ -1338,7 +1353,7 @@ static gboolean
 fstab_user_permission(Mount *m)
 	{
 	struct stat my_stat;
-	
+
 	stat(m->device, &my_stat);
 	if (   strstr(m->options, "user")
 		|| (strstr(m->options, "owner") && my_stat.st_uid == uid)
@@ -1361,7 +1376,7 @@ fix_fstab_mountable_changed(FSmon *fs)
 		{
 		fs->fstab_mounting = FALSE;
 		return TRUE;
-		}		
+		}
 	return FALSE;
 	}
 
@@ -1387,8 +1402,8 @@ fs_config_load(gchar *arg)
 	{
 	static FSmon	*fs_prev;
 	FSmon			*fs;
-	gchar			*label;
-	gchar			config[32], item[CFG_BUFSIZE], dir[CFG_BUFSIZE];
+	gchar			*cut_label, *cut_dir;
+	gchar			config[32], item[CFG_BUFSIZE];
 	gchar			name[64], item1[CFG_BUFSIZE];
 	gint 			n;
 
@@ -1435,13 +1450,14 @@ fs_config_load(gchar *arg)
 		}
 	else
 		{
-		if ((label = gkrellm_cut_quoted_string(arg, &arg)) != NULL)
+		if (   (cut_label = gkrellm_cut_quoted_string(arg, &arg)) != NULL
+			&& (cut_dir = gkrellm_cut_quoted_string(arg, &arg)) != NULL
+	       )
 			{
 			fs = g_new0(FSmon, 1);
-			gkrellm_locale_dup_string(&fs->label, label, &fs->label_shadow);
-			dir[0] = '\0';
+			gkrellm_locale_dup_string(&fs->label, cut_label,&fs->label_shadow);
 
-			sscanf(arg, "%s %d %d %d %d %d",  dir, &fs->fstab_mounting,
+			sscanf(arg, "%d %d %d %d %d", &fs->fstab_mounting,
 					&fs->secondary, &fs->show_if_mounted,
 					&fs->label_is_data, &fs->ejectable);
 			if (fs->fstab_mounting > 1)		/* pre 2.0.0 config fix */
@@ -1452,7 +1468,7 @@ fs_config_load(gchar *arg)
 				fs->fstab_mounting = fs->show_if_mounted = FALSE;
 			if (fs->secondary)
 				have_secondary_panels = TRUE;
-			fs->mount.directory = g_strdup(dir);
+			fs->mount.directory = g_strdup(cut_dir);
 			fs->restore_label = fs->label_is_data;
 
 			fix_fstab_mountable_changed(fs);
@@ -1493,7 +1509,7 @@ static GtkTreeSelection	*selection;
 
 static GtkWidget
 				*label_entry,
-				*dir_combo,
+				*dir_combo_box,
 				*mount_entry,
 				*umount_entry,
 				*mounting_button,
@@ -1506,7 +1522,7 @@ static GtkWidget
 
 static GtkWidget	*alert_button;
 
-static GtkWidget	*data_format_combo;
+static GtkWidget	*data_format_combo_box;
 
 static gboolean	(*original_row_drop_possible)();
 
@@ -1664,14 +1680,17 @@ cb_alert_config(GkrellmAlert *ap, FSmon *fs)
   |  fstab entries and accordingly set sensitivity of the mounting_button.
   */
 static void
-cb_combo_changed(GtkWidget *widget)
+cb_combo_changed(GtkComboBox *widget, gpointer user_data)
 	{
 	Mount	*m;
 	gchar	*s;
+	GtkWidget *entry;
 
 	if (!mounting_supported || _GK.client_mode)
 		return;
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(dir_combo)->entry));
+
+	entry = gtk_bin_get_child(GTK_BIN(dir_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 	m = in_fstab_list(s);
 	if (m && (fstab_user_permission(m) || uid == 0))
 		{
@@ -1763,7 +1782,7 @@ static void
 reset_entries(gboolean level0)
 	{
 	gtk_entry_set_text(GTK_ENTRY(label_entry), "");
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dir_combo)->entry), "");
+	gtk_combo_box_set_active(GTK_COMBO_BOX(dir_combo_box), -1);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(secondary_button), level0);
 	if (mounting_button)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mounting_button),FALSE);
@@ -1855,8 +1874,8 @@ cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(secondary_button),
 					secondary);
 	gtk_entry_set_text(GTK_ENTRY(label_entry), fs->label);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dir_combo)->entry),
-				fs->mount.directory);
+	gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(dir_combo_box))),
+			fs->mount.directory);
 	if (show_button)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_button),
 					fs->show_if_mounted);
@@ -1970,6 +1989,7 @@ add_cb(GtkWidget *widget)
 	gchar			*label;
 	gint			secondary, *indices;
 	gboolean		a, b, err = FALSE;
+	GtkWidget		*entry;
 
 	fs = g_new0(FSmon, 1);
 	fs->launch_mount.command = g_strdup("");
@@ -1978,8 +1998,10 @@ add_cb(GtkWidget *widget)
 
 	label = gkrellm_gtk_entry_get_text(&label_entry);
 	gkrellm_locale_dup_string(&fs->label, label, &fs->label_shadow);
-	fs->mount.directory =
-		g_strdup(gkrellm_gtk_entry_get_text(&(GTK_COMBO(dir_combo)->entry)));
+
+	entry = gtk_bin_get_child(GTK_BIN(dir_combo_box));
+	fs->mount.directory = g_strdup(gkrellm_gtk_entry_get_text(&entry));
+
 	if (show_button)
 		fs->show_if_mounted = GTK_TOGGLE_BUTTON(show_button)->active;
 	if (mounting_button)
@@ -2100,8 +2122,10 @@ cb_data_format(GtkWidget *widget, gpointer data)
 	GkrellmDecal	*d;
 	gchar			*s, buf[256];
 	gint			h_data, h_label;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(data_format_combo)->entry));
+	entry = gtk_bin_get_child(GTK_BIN(data_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 
 	/* In case Pango markup tags, don't accept line unless valid markup.
 	|  Ie, markup like <span ...> xxx </span> or <b> xxx </b>
@@ -2241,7 +2265,7 @@ create_fs_panels_page(GtkWidget *vbox)
 	GtkWidget				*scrolled;
 	GtkTreeModel			*model;
 	GtkCellRenderer			*renderer;
-	GList					*list, *combo_list;
+	GList					*list;
 
 
 	hbox = gtk_hbox_new(FALSE, 0);
@@ -2266,18 +2290,16 @@ create_fs_panels_page(GtkWidget *vbox)
 #endif
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
 				GTK_SHRINK, GTK_SHRINK, 2, 1);
-	dir_combo = gtk_combo_new();
-	gtk_table_attach_defaults(GTK_TABLE(table), dir_combo, 1, 2, 1, 2);
-	combo_list = NULL;
+	dir_combo_box = gtk_combo_box_entry_new_text();
+	gtk_table_attach_defaults(GTK_TABLE(table), dir_combo_box, 1, 2, 1, 2);
 	for (list = fstab_list; list; list = list->next)
-		combo_list = g_list_append(combo_list,
-					((Mount *)list->data)->directory);
-	gtk_combo_set_popdown_strings( GTK_COMBO(dir_combo), combo_list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(dir_combo), TRUE);
-	g_list_free(combo_list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dir_combo)->entry), "");
-	g_signal_connect(G_OBJECT(GTK_COMBO(dir_combo)->entry),
-				"changed", GTK_SIGNAL_FUNC (cb_combo_changed), NULL);
+		{
+		gtk_combo_box_append_text(GTK_COMBO_BOX(dir_combo_box),
+				((Mount *)list->data)->directory);
+		}
+	gtk_combo_box_set_active(GTK_COMBO_BOX(dir_combo_box), -1);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(dir_combo_box)),
+			"changed", G_CALLBACK(cb_combo_changed), NULL);
 
 	vbox1 = gtk_vbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox1, FALSE, FALSE, 0);
@@ -2487,7 +2509,6 @@ fs_tab_create(GtkWidget *tab_vbox)
 	GtkWidget		*tabs;
 	GtkWidget		*vbox, *vbox1;
 	GtkWidget		*text;
-	GList			*list;
 	gint			i;
 
 	row_reference = NULL;
@@ -2541,19 +2562,14 @@ fs_tab_create(GtkWidget *tab_vbox)
 	vbox1 = gkrellm_gtk_category_vbox(vbox,
 				_("Format String for Panel Labels"),
 				4, 0, TRUE);
-	data_format_combo = gtk_combo_new();
-	gtk_box_pack_start(GTK_BOX(vbox1), data_format_combo, FALSE, FALSE, 2);
-	list = NULL;
-	list = g_list_append(list, data_format);
-	list = g_list_append(list, DEFAULT_DATA_FORMAT);
-	list = g_list_append(list, _("$t - $u used"));
-	list = g_list_append(list, _("$t - $U"));
-	gtk_combo_set_popdown_strings(GTK_COMBO(data_format_combo), list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(data_format_combo), TRUE);
-	g_list_free(list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(data_format_combo)->entry),
-			data_format);
-	g_signal_connect(G_OBJECT(GTK_COMBO(data_format_combo)->entry), "changed",
+	data_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(vbox1), data_format_combo_box, FALSE, FALSE, 2);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(data_format_combo_box), data_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(data_format_combo_box), DEFAULT_DATA_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(data_format_combo_box), ALT1_DATA_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(data_format_combo_box), ALT2_DATA_FORMAT);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(data_format_combo_box), 0);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(data_format_combo_box)), "changed",
 			G_CALLBACK(cb_data_format), NULL);
 
 /* --Info tab */
