@@ -1,22 +1,34 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2008 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "configure.h"
@@ -33,6 +45,7 @@
 #define	DEBUG_POSITION		0x4000
 #define	DEBUG_BATTERY		0x8000
 #define	DEBUG_CHART_TEXT	0x10000
+#define	DEBUG_PLUGIN		0x20000
 
 #define	GKRELLM_FLOAT_FACTOR	1000.0
 
@@ -64,7 +77,9 @@ struct	GkrellmConfig
 	gint		demo;
 	gint		test;
 	gboolean	nolock;
-	gboolean	without_libsensors;
+	gboolean	without_libsensors;		/* transition override */
+	gboolean	use_acpi_battery;		/* transition override */
+	gboolean	config_clean;
 
 	gint		up_minutes;
 	gint		base_uptime;
@@ -313,6 +328,8 @@ struct	GkrellmConfig
 
 extern struct GkrellmConfig  _GK;
 
+extern GList		*gkrellm_monitor_list;
+
 extern struct tm	gkrellm_current_tm;
 extern gint			gkrellm_w_display,
 					gkrellm_h_display,
@@ -374,7 +391,7 @@ void	gkrellm_chart_setup(void);
 
 void	gkrellm_build(void);
 void	gkrellm_theme_config(void);
-void	gkrellm_load_user_config(gboolean);
+void	gkrellm_load_user_config(GkrellmMonitor *mon_only, gboolean);
 void	gkrellm_save_user_config(void);
 void	gkrellm_save_theme_config(void);
 void	gkrellm_load_theme_config(void);
@@ -445,7 +462,8 @@ void		gkrellm_sensor_reset_location(gpointer sr);
 void		gkrellm_sensors_rebuild(gboolean do_temp, gboolean do_fan,
 						gboolean do_volt);
 void		gkrellm_sensors_model_update(void);
-void		gkrellm_sensors_interface_remove(gint interface);
+void		gkrellm_sensors_interface_remove(gint _interface);
+void		gkrellm_sensors_sysdep_option(gchar *, gchar *, void (*func)());
 
 /* pixops */
 void		gkrellm_border_adjust(GkrellmBorder *border,
