@@ -1,29 +1,43 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2008 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
 #include "gkrellm-private.h"
 #include "gkrellm-sysdeps.h"
+#include "log-private.h"
 
-#if GTK_CHECK_VERSION(2,4,0)
+// GTK+-win32 automatically uses the application-icon
+#if GTK_CHECK_VERSION(2,4,0) && !defined(WIN32)
 #include "icon.xpm"
 #endif
 
@@ -133,17 +147,15 @@ load_font(gchar *font_string, PangoFontDescription **gk_font,
 
 	if (font_string)
 		font_desc = pango_font_description_from_string(font_string);
-	if (_GK.debug_level & DEBUG_GUI)
-		printf("load_font: %s %p\n", font_string, font_desc);
+	gkrellm_debug(DEBUG_GUI, "load_font: %s %p\n", font_string, font_desc);
 
 	if (!font_desc)
 		{
 		for (i = 0; !font_desc && i < N_FALLBACK_FONTS; ++i)
 			{
 			font_desc = pango_font_description_from_string(fallback_fonts[i]);
-			if (_GK.debug_level & DEBUG_GUI)
-				printf("load_font trying fallback: %s\n",
-							fallback_fonts[i]);
+			gkrellm_debug(DEBUG_GUI, "load_font trying fallback: %s\n",
+						fallback_fonts[i]);
 			}
 		}
 	if (*gk_font)
@@ -222,7 +234,7 @@ setup_colors()
 
 	/* Set up the depth 1 GCs
 	*/
-	/* printf("white pixel = %ld\n", _GK.white_color.pixel); */
+	/* g_print("white pixel = %ld\n", _GK.white_color.pixel); */
 	if (_GK.bit1_GC == NULL)
 		{
 		GdkBitmap	*dummy_bitmap;
@@ -265,7 +277,8 @@ set_or_save_position(gint save)
 			gdk_window_get_position(top_window->window,
 					&_GK.x_position, &_GK.y_position);
 			}
-		if (   (_GK.x_position != x_last || _GK.y_position != y_last)
+		if (   !_GK.no_config
+		    && (_GK.x_position != x_last || _GK.y_position != y_last)
 			&& (f = fopen(path, "w")) != NULL
 		   )
 			{
@@ -273,8 +286,7 @@ set_or_save_position(gint save)
 			y_last = _GK.y_position;
 			fprintf(f, "%d %d\n", _GK.x_position, _GK.y_position);
 			fclose(f);
-			if (_GK.debug_level & DEBUG_POSITION)
-				printf("save_position: %d %d\n", x_last, y_last);
+			gkrellm_debug(DEBUG_POSITION, "save_position: %d %d\n", x_last, y_last);
 			}
 		save_position_countdown = 0;
 		}
@@ -294,8 +306,7 @@ set_or_save_position(gint save)
 				_GK.y_position = y_last = y;
 				_GK.position_valid = TRUE;
 				gdk_window_move(gtree.window->window, x, y);
-				if (_GK.debug_level & DEBUG_POSITION)
-					printf("startup_position moveto %d %d (valid)\n", x, y);
+				gkrellm_debug(DEBUG_POSITION, "startup_position moveto %d %d (valid)\n", x, y);
 				}
 			}
 		}
@@ -601,8 +612,7 @@ top_frame_button_release(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	if (!no_transparency)
 		gkrellm_winop_apply_rootpixmap_transparency();
 	moving_gkrellm = FALSE;
-	if (_GK.debug_level & DEBUG_POSITION)
-		printf("gkrellm moveto: x_pos=%d y_pos=%d\n",
+	gkrellm_debug(DEBUG_POSITION, "gkrellm moveto: x_pos=%d y_pos=%d\n",
 				_GK.x_position, _GK.y_position);
 	}
 
@@ -614,8 +624,8 @@ top_frame_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	time_t			time_check;
 
 	gdk_window_get_pointer(NULL, &x_pointer, &y_pointer, &m);
-	gdk_window_get_geometry(NULL, NULL, NULL, &_GK.w_display,
-				&_GK.h_display, NULL);
+	_GK.w_display = gdk_screen_get_width(gdk_screen_get_default());
+	_GK.h_display = gdk_screen_get_height(gdk_screen_get_default());
 	gkrellm_width = _GK.chart_width
 				+ _GK.frame_left_width + _GK.frame_right_width;
 	gkrellm_height = _GK.monitor_height + _GK.total_frame_height;
@@ -1178,7 +1188,7 @@ gkrellm_pack_side_frames(void)
 		{
 		rf_image = gtk_image_new_from_pixmap(gtree.frame_right_pixmap, NULL);
 		gtk_box_pack_start(GTK_BOX(gtree.right_vbox), rf_image,
-					FALSE, FALSE, 0);	
+					FALSE, FALSE, 0);
 		gtk_widget_show(rf_image);
 		}
 	else
@@ -1198,9 +1208,9 @@ gkrellm_pack_side_frames(void)
 			{
 			gdk_window_move(gtree.window->window,
 							_GK.x_position, _GK.y_position);
-			if (_GK.debug_level & DEBUG_POSITION)
-				printf("pack moveto %d %d=y_position (y_gkrell=%d y_bot=%d)\n",
-						_GK.x_position, _GK.y_position, y_gkrell, y_bottom);
+			gkrellm_debug(DEBUG_POSITION,
+				"pack moveto %d %d=y_position (y_gkrell=%d y_bot=%d)\n",
+				_GK.x_position, _GK.y_position, y_gkrell, y_bottom);
 			}
 		y_pack = -1;
 		}
@@ -1217,9 +1227,9 @@ gkrellm_pack_side_frames(void)
 			if (_GK.position_valid)
 				{
 				gdk_window_move(gtree.window->window, _GK.x_position, y_pack);
-				if (_GK.debug_level & DEBUG_POSITION)
-					printf("pack moveto %d %d=y_pack (y_gkrell=%d y_bot=%d)\n",
-							_GK.x_position, y_pack, y_gkrell, y_bottom);
+				gkrellm_debug(DEBUG_POSITION,
+					"pack moveto %d %d=y_pack (y_gkrell=%d y_bot=%d)\n",
+					_GK.x_position, y_pack, y_gkrell, y_bottom);
 				}
 			}
 		/* If GKrellM bottom edge was <= screen bottom, then move to make
@@ -1232,9 +1242,9 @@ gkrellm_pack_side_frames(void)
 			if (_GK.position_valid)
 				{
 				gdk_window_move(gtree.window->window, _GK.x_position, y_pack);
-				if (_GK.debug_level & DEBUG_POSITION)
-					printf("pack moveto %d %d=y_pack (y_gkrell=%d on_bottom)\n",
-							_GK.x_position, y_pack, y_gkrell);
+				gkrellm_debug(DEBUG_POSITION,
+					"pack moveto %d %d=y_pack (y_gkrell=%d on_bottom)\n",
+					_GK.x_position, y_pack, y_gkrell);
 				}
 			}
 		}
@@ -1292,8 +1302,8 @@ fix_edges()
 		gdk_window_move(gtree.window->window, x, y);
 		if (y != _GK.y_position)
 			y_pack = y;
-		if (_GK.debug_level & DEBUG_POSITION)
-			printf("fix_edges: %d %d (y_pos=%d)\n", x, y, _GK.y_position);
+		gkrellm_debug(DEBUG_POSITION,
+			"fix_edges: %d %d (y_pos=%d)\n", x, y, _GK.y_position);
 		}
 	}
 
@@ -1318,7 +1328,7 @@ gkrellm_render_spacer(GkrellmSpacer *spacer, gint y_src, gint h_src,
 	   )
 		{
 		if (spacer->height > 0)
-			fprintf(stderr, "Bad image size for spacer or bg_chart.\n");
+			g_warning("Bad image size for spacer or bg_chart.\n");
 		return FALSE;
 		}
 	piximage.border = spacer->piximage->border;
@@ -1485,14 +1495,14 @@ static void
 create_widget_tree()
 	{
 	gchar		*title;
-#if GTK_CHECK_VERSION(2,4,0)
+#if GTK_CHECK_VERSION(2,4,0) && !defined(WIN32)
 	GdkPixbuf	*icon;
 #endif
 
 	gtree.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_widget_set_name(gtree.window, PACKAGE);
 
-#if GTK_CHECK_VERSION(2,4,0)
+#if GTK_CHECK_VERSION(2,4,0) && !defined(WIN32)
 	icon = gdk_pixbuf_new_from_xpm_data((const gchar **) icon_xpm);
 	gtk_window_set_default_icon(icon);
 #endif
@@ -1572,7 +1582,7 @@ cb_client_event(GtkWidget  *widget, GdkEventClient *event, gpointer data)
 	if (!atom_gkrellm_read_theme)
 		atom_gkrellm_read_theme = gdk_atom_intern("_GKRELLM_READ_THEME",FALSE);
 
-	if (event->message_type == atom_gkrellm_read_theme) 
+	if (event->message_type == atom_gkrellm_read_theme)
 		gkrellm_read_theme_event(NULL);
 	return FALSE;
 	}
@@ -1616,8 +1626,8 @@ cb_configure_notify(GtkWidget *widget, GdkEventConfigure *ev, gpointer data)
 	gint        x, y, w, h, w_gkrellm, h_gkrellm;
 	static gint	x_prev, y_prev, w_prev, h_prev;
 
-	gdk_window_get_geometry(NULL, NULL, NULL, &_GK.w_display,
-				&_GK.h_display, NULL);
+	_GK.w_display = gdk_screen_get_width(gdk_screen_get_default());
+	_GK.h_display = gdk_screen_get_height(gdk_screen_get_default());
 
 #if !defined(WIN32)
 	gdk_window_get_position(widget->window, &x, &y);
@@ -1672,12 +1682,16 @@ cb_configure_notify(GtkWidget *widget, GdkEventConfigure *ev, gpointer data)
 		if (y >= 0 && y < _GK.h_display - 5 && y != y_pack)
 			_GK.y_position = y;
 		_GK.position_valid = TRUE;
-		if (!moving_gkrellm && (_GK.debug_level & DEBUG_POSITION))
-			printf("configure-event: x_pos=%d y_pos=%d x=%d y=%d y_pack=%d\n",
-						_GK.x_position, _GK.y_position, x, y, y_pack);
+		if (!moving_gkrellm)
+			gkrellm_debug(DEBUG_POSITION,
+				"configure-event: x_pos=%d y_pos=%d x=%d y=%d y_pack=%d\n",
+				_GK.x_position, _GK.y_position, x, y, y_pack);
+		}
+	else
+		{
+		gkrellm_debug(DEBUG_POSITION,
+			"locked configure-event: x=%d y=%d\n", x, y);
 		}
-	else if (_GK.debug_level & DEBUG_POSITION)
-			printf("locked configure-event: x=%d y=%d\n", x, y);
 
 	if (size_change || position_change)
 		gkrellm_winop_update_struts();
@@ -1761,46 +1775,59 @@ cb_destroy_event()
 	gtk_main_quit();
 	}
 
-static gchar	*usage_string =
+static gchar	*usage_string[] =
+{
 N_("usage: gkrellm [options]\n"
-"options:\n"
-"   -t, --theme theme_dir    Select a theme directory.\n"
-"   -g, --geometry +x+y      Position the window on the screen.\n"
-"       --wm                 Allow window manager decorations.\n"
-"       --m2                 Left button side frame shading (for 2 btn mice).\n"
-"       --nt                 No transparency.\n"
-"   -w, --withdrawn          Draw GKrellM in withdrawn mode.\n"
-"   -c, --config suffix      Use alternate config files generated by\n"
-"                            appending \"suffix\" to config file names.\n"
-"   -f, --force-host-config  Creates config files generated by appending the\n"
-"                            hostname to config file names.  Subsequent runs\n"
-"                            automatically will use these configs unless a\n"
-"                            specific config is specified with --config.\n"
-"                            This is a convenience for allowing remote runs\n"
-"                            with independent configs in a shared home dir\n"
-"                            and for the hostname to be in the window title.\n"
-"                            This option has no effect in client mode.\n"
-"   -s, --server hostname    Run in client mode: connect to \"hostname\" and\n"
-"                            read monitor data from a gkrellmd server.\n"
-"   -P, --port server_port   Use \"server_port\" for the server connection.\n"
-"       --nc                 No config mode prevents configuration changes.\n"
-"   -p, --plugin plugin.so   While developing, load your plugin under test.\n"
-"       --demo               Force enabling of many monitors so themers can\n"
-"                            see everything. All config saving is inhibited.\n"
-"   -v, --version            Print GKrellM version number and exit.\n"
-"   -d, --debug-level n      Turn debugging on for selective code sections.\n"
+   "options:\n"),
+N_("   -t, --theme theme_dir    Select a theme directory.\n"),
+N_("   -g, --geometry +x+y      Position the window on the screen.\n"),
+N_("       --wm                 Allow window manager decorations.\n"),
+N_("       --m2                 Left button side frame shading (for 2 btn mice).\n"),
+N_("       --nt                 No transparency.\n"),
+N_("   -w, --withdrawn          Draw GKrellM in withdrawn mode.\n"),
+N_("   -c, --config suffix      Use alternate config files generated by\n"
+   "                            appending \"suffix\" to config file names.\n"),
+N_("   -f, --force-host-config  Creates config files generated by appending the\n"
+   "                            hostname to config file names.  Subsequent runs\n"
+   "                            automatically will use these configs unless a\n"
+   "                            specific config is specified with --config.\n"
+   "                            This is a convenience for allowing remote runs\n"
+   "                            with independent configs in a shared home dir\n"
+   "                            and for the hostname to be in the window title.\n"
+   "                            This option has no effect in client mode.\n"),
+N_("   -s, --server hostname    Run in client mode: connect to \"hostname\" and\n"
+   "                            read monitor data from a gkrellmd server.\n"),
+N_("   -P, --port server_port   Use \"server_port\" for the server connection.\n"),
+N_("       --nc                 No config mode prevents configuration changes.\n"),
+N_("       --config-clean       Clean out unused configs on config write.\n"),
+N_("       --nolock             Allow multiple gkrellm instances.\n"),
+N_("   -p, --plugin plugin.so   While developing, load your plugin under test.\n"),
+N_("       --demo               Force enabling of many monitors so themers can\n"
+   "                            see everything. All config saving is inhibited.\n"),
+   "   -l, --logfile path       Enable error/debugging to a log file.\n",
+N_("   -v, --version            Print GKrellM version number and exit.\n"),
+N_("   -d, --debug-level n      Turn debugging on for selective code sections.\n"),
 
-"\ndebug-level numbers are (bitwise OR to debug multiple sections):\n"
-"     0x10  mail\n"
-"     0x20  net\n"
-"     0x40  timer button\n"
-"     0x80  sensors\n\n"
-);
+N_("\ndebug-level numbers are (bitwise OR to debug multiple sections):\n"
+"     0x10    mail\n"
+"     0x20    net\n"
+"     0x40    timer button\n"
+"     0x80    sensors\n"
+"     0x800   inet\n"
+"     0x1000  dump gkrellmd server data\n"
+"     0x2000  GUI\n"
+"     0x8000  battery\n"
+"     0x20000 plugin \n"
+"\n")
+};
 
 static void
 usage()
 	{
-	g_print(_(usage_string));
+	gint	i;
+
+	for (i = 0; i < sizeof(usage_string) / sizeof(gchar *); ++i)
+		g_print(_(usage_string[i]));
 	}
 
 
@@ -1831,7 +1858,7 @@ gkrellm_build()
 
 	gkrellm_theme_config();
 	if (first_create)
-		gkrellm_load_user_config(TRUE);
+		gkrellm_load_user_config(NULL, TRUE);
 	setup_colors();
 	setup_fonts();
 	gkrellm_load_theme_piximages();
@@ -1935,26 +1962,11 @@ load_builtin_monitors()
 	add_builtin(gkrellm_init_uptime_monitor());
 	}
 
-
-void
-gkrellm_print(const gchar *string)
-	{
-	gchar	*s;
-
-	s = g_locale_from_utf8(string, -1, NULL, NULL, NULL);
-	if (s)
-		{
-		fputs(s, stdout);
-		g_free(s);
-		}
-	else
-		fputs(string, stdout);
-	}
-
 static void
 gkrellm_exit(gint exit_code)
 	{
 	gkrellm_sys_main_cleanup();
+	gkrellm_log_cleanup();
 	exit(exit_code);
 	}
 
@@ -2023,9 +2035,11 @@ gkrellm_abort(gint sig)
 static void
 setup_signal_handler(void)
 	{
+#ifndef WIN32
 	signal(SIGPIPE, SIG_IGN);
-	signal(SIGINT, _signal_quit);
 	signal(SIGQUIT, _signal_quit);
+#endif
+	signal(SIGINT, _signal_quit);
 	signal(SIGTERM, _signal_quit);
 	}
 
@@ -2036,22 +2050,38 @@ main(gint argc, gchar **argv)
 	gint		i;
 	gchar		*s;
 
-	gkrellm_sys_main_init();
-
 #ifdef ENABLE_NLS
 	gtk_set_locale();
 #endif
-
 	g_thread_init(NULL);
-
 	gtk_init(&argc, &argv);		/* Will call gdk_init() */
-
+	gkrellm_log_init();
 	gtk_widget_push_colormap(gdk_rgb_get_colormap());
 
 #ifdef ENABLE_NLS
 #ifdef LOCALEDIR
+#if defined(WIN32)
+	/*
+	Prepend app install path to relative locale dir, don't rely on CWD being correct
+	*/
+	if (!G_IS_DIR_SEPARATOR(LOCALEDIR[0]))
+		{
+ 		gchar* locale_dir;
+		locale_dir = g_win32_get_package_installation_subdirectory(NULL, NULL, LOCALEDIR);
+		if (locale_dir != NULL)
+			{
+			bindtextdomain(PACKAGE, locale_dir);
+			g_free(locale_dir);
+			}
+		}
+	else
+		{
+		bindtextdomain(PACKAGE, LOCALEDIR);
+		}
+#else
 	bindtextdomain(PACKAGE, LOCALEDIR);
-#endif
+#endif /* !WIN32 */
+#endif /* LOCALEDIR */
 	textdomain(PACKAGE);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 #endif	/* ENABLE_NLS */
@@ -2065,8 +2095,6 @@ main(gint argc, gchar **argv)
 	signal(SIGSEGV, gkrellm_abort);
 	signal(SIGABRT, gkrellm_abort);
 
-	g_set_print_handler(gkrellm_print);
-
 	for (i = 1; i < argc; ++i)
 		{
 		s = argv[i];
@@ -2113,6 +2141,8 @@ main(gint argc, gchar **argv)
 			_GK.no_config = TRUE;
 		else if ((!strcmp(s, "debug-level") || !strcmp(s, "d")) && i < argc-1)
 			_GK.debug_level = (gint) strtoul(argv[++i], NULL, 0);
+		else if ((!strcmp(s, "logfile") || !strcmp(s, "l")) && i < argc-1)
+			gkrellm_log_set_filename(argv[++i]);
 		else if (!strncmp(s, "debug", 5))
 			{
 			if (s[5] != '\0')
@@ -2122,15 +2152,22 @@ main(gint argc, gchar **argv)
 			}
 		else if (!strcmp(s, "nolock"))
 			_GK.nolock = TRUE;
-		else if (!strcmp(s, "without-libsensors"))
+		else if (!strcmp(s, "without-libsensors"))		/* temporary */
 			_GK.without_libsensors = TRUE;
+		else if (!strcmp(s, "use-acpi-battery"))		/* temporary */
+			_GK.use_acpi_battery = TRUE;
+		else if (!strcmp(s, "config-clean"))
+			{
+			_GK.config_clean = TRUE;
+			_GK.config_modified = TRUE;
+			}
 		else if (!strcmp(s, "demo"))
 			++_GK.demo;
 		else if (!strcmp(s, "test"))
 			_GK.test += 1;
 		else if (!strcmp(s, "version") || !strcmp(s, "v"))
 			{
-			printf("%s %d.%d.%d%s\n", PACKAGE, GKRELLM_VERSION_MAJOR,
+			g_print("%s %d.%d.%d%s\n", PACKAGE, GKRELLM_VERSION_MAJOR,
 					GKRELLM_VERSION_MINOR, GKRELLM_VERSION_REV,
 					GKRELLM_EXTRAVERSION);
 			exit(0);
@@ -2147,17 +2184,24 @@ main(gint argc, gchar **argv)
 			exit(0);
 			}
 		}
-	gdk_window_get_geometry(NULL /*gdk_root_parent*/, &i, &i, &_GK.w_display,
-				&_GK.h_display, &i);
 
-	if (_GK.debug_level)
-		printf("--- GKrellM %d.%d.%d ---\n", GKRELLM_VERSION_MAJOR,
-				GKRELLM_VERSION_MINOR, GKRELLM_VERSION_REV);
+	if (_GK.debug_level > 0)
+		g_debug("--- GKrellM %d.%d.%d ---\n", GKRELLM_VERSION_MAJOR,
+			GKRELLM_VERSION_MINOR, GKRELLM_VERSION_REV);
+
+	gkrellm_sys_main_init(); //FIXME: call this later or earlier?
+
+	_GK.w_display = gdk_screen_get_width(gdk_screen_get_default());
+	_GK.h_display = gdk_screen_get_height(gdk_screen_get_default());
+
 	if (_GK.server && !gkrellm_client_mode_connect())
-		exit(0);
+		{
+		gkrellm_exit(0);
+		return 0;
+		}
 
 	check_gkrellm_directories();
-	gkrellm_load_user_config(FALSE);		
+	gkrellm_load_user_config(NULL, FALSE);
 	decorated = (_GK.command_line_decorated || _GK.decorated);
 	if (   _GK.command_line_plugin || _GK.command_line_theme
 		|| _GK.debug_level > 0 || _GK.debug > 0 || _GK.nolock
@@ -2231,33 +2275,10 @@ main(gint argc, gchar **argv)
 
 	gkrellm_start_timer(_GK.update_HZ);
 	setup_signal_handler();
-	gtk_main ();
+	gtk_main();
+
 	gkrellm_save_all();
 	gkrellm_exit(0);
 
 	return 0;
 	}
-
-
-#if defined(WIN32) && defined(_WINDOWS)
-int APIENTRY WinMain(HINSTANCE	hInstance,
-					HINSTANCE	hPrevInstance,
-					LPSTR		lpCmdLine,
-					int			nCmdShow)
-	{
-	int		argc;
-	gint	i, ret;
-	gchar	*s, *sm;
-	gchar	**argv;
-	gchar	*cmd;
-
-	// need to get exe too
-	cmd = GetCommandLine();
-	g_shell_parse_argv(cmd, &argc, &argv, NULL);
-
-	ret = main(argc, argv);
-    g_strfreev(argv);
-	return ret;
-	}
-#endif
-
