@@ -1,22 +1,34 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2008 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -26,7 +38,7 @@
 #include <gmodule.h>
 
 #if defined(WIN32)
-#include "win32-plugin.h"
+	#include "win32-plugin.h"
 #endif
 
 /* ======================================================================= */
@@ -564,6 +576,93 @@ gkrellm_plugin_debug(void)
 	return _GK.debug;
 	}
 
+/* ======================================================================= */
+typedef struct
+	{
+	GkrellmMonitor	*mon_plugin,
+					*mon_target;
+	gchar			key;
+	gchar			*(*func)(gchar key, gchar *which);
+	gint			id;
+	}
+	ExportLabel;
+
+static GList	*export_label_list;
+
+gint
+gkrellm_plugin_export_label(GkrellmMonitor *mon_plugin, gchar *mon_name,
+			gchar key, gchar *(*func)(gchar key, gchar *which))
+	{
+	GkrellmMonitor	*mon;
+	ExportLabel		*el;
+	GList			*list;
+	static gint		id;
+
+	if (!mon_name || !mon_plugin || !func || !key)
+		return -1;
+	for (list = gkrellm_monitor_list; list; list = list->next)
+		{
+		mon = (GkrellmMonitor *) list->data;
+		if (mon->name && (_GK.debug_level & DEBUG_PLUGIN))
+			g_print("    %s %s\n", mon->name, mon_name);
+		if (mon->name && !strcmp(mon->name, mon_name))
+			break;
+		}
+	if (!list)
+		return -1;
+
+	el = g_new0(ExportLabel, 1);
+	export_label_list = g_list_append(export_label_list, el);
+	el->mon_target = mon;
+	el->mon_plugin = mon_plugin;
+	el->key = key;
+	el->func = func;
+	el->id = ++id;
+
+	return id;
+	}
+
+void
+gkrellm_plugin_cancel_label(GkrellmMonitor *mon_plugin, gint id)
+	{
+	GList		*list;
+	ExportLabel	*el;
+
+	if (!mon_plugin || id < 1)
+		return;
+	for (list = export_label_list; list; list = list->next)
+		{
+		el = (ExportLabel *) list->data;
+		if (el->id == id)
+			{
+			export_label_list = g_list_remove(export_label_list, el);
+			g_free(el);
+			break;
+			}
+		}
+	}
+
+gchar *
+gkrellm_plugin_get_exported_label(GkrellmMonitor *mon, gchar key, gchar *which)
+	{
+	GList		*list;
+	ExportLabel	*el;
+	gchar		*result = NULL;
+
+	for (list = export_label_list; list; list = list->next)
+		{
+		el = (ExportLabel *) list->data;
+		if (   el->mon_target == mon && el->key == key
+			&& el->mon_plugin->privat->enabled && el->func
+		   )
+			{
+			result = (*el->func)(key, which);
+			break;
+			}
+		}
+	return result;
+	}
+
 /* ======================================================================= */
 
 static GkrellmMonitor		*place_plugin;
@@ -811,11 +910,23 @@ install_plugin(gchar *plugin_name)
 	mon_tmp.name = g_strdup(plugin_name);
 	gkrellm_record_state(INIT_MONITOR, &mon_tmp);
 
-#if !defined(WIN32)
-	m = (*init_plugin)();
-#else
-	m = (*init_plugin)(callbacks);
+#if defined(WIN32)
+	{
+		win32_plugin_callbacks ** plugin_cb = NULL;
+
+		if (!g_module_symbol(module, "callbacks", (gpointer) &plugin_cb))
+		{
+			snprintf(buf, sizeof(buf), _("\tError: %s\n"), g_module_error());
+			plugin_log(buf, NULL);
+			g_module_close(module);
+			return NULL;
+		}
+		*plugin_cb = &gkrellm_callbacks;
+	}
 #endif
+
+	m = (*init_plugin)();
+
 	_GK.no_messages = FALSE;
 
 	g_free(mon_tmp.name);
@@ -924,6 +1035,7 @@ enable_plugin(GkrellmMonitor *plugin)
 	gtk_widget_show(mp->vbox);
 	gtk_widget_show(mp->main_vbox);
 	mp->enabled = TRUE;
+	gkrellm_load_user_config(plugin, TRUE);
 	gkrellm_build();
 	gkrellm_add_plugin_config_page(plugin);
 	plugin_enable_list_modified = TRUE;
@@ -961,7 +1073,7 @@ save_plugins_placement_file(void)
 	GkrellmMonprivate	*mp;
 	gchar				*path;
 	
-	if (!plugin_placement_modified || _GK.demo)
+	if (!plugin_placement_modified || _GK.demo || _GK.no_config)
 		return;
 	path = gkrellm_make_config_file_name(gkrellm_homedir(),
 				PLUGIN_PLACEMENT_FILE);
