@@ -1,22 +1,34 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2008 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -24,8 +36,6 @@
 #include "gkrellm-sysdeps.h"
 
 
-static gboolean (*mbmon_check_func)();
-
 
 #if defined(__linux__)
 #include "sysdeps/linux.c"
@@ -43,10 +53,17 @@ static gboolean (*mbmon_check_func)();
 #include "sysdeps/sensors-common.c"
 #endif
 
+#if defined(__DragonFly__)
+#include "sysdeps/dragonfly.c"
+#include "sysdeps/bsd-common.c"
+#include "sysdeps/sensors-common.c"
+#endif
+
 #if defined(__NetBSD__)
 #include "sysdeps/netbsd.c"
 #include "sysdeps/bsd-net-open.c"
 #include "sysdeps/bsd-common.c"
+#include "sysdeps/sensors-common.c"
 #endif
 
 #if defined(__OpenBSD__)
@@ -63,7 +80,17 @@ static gboolean (*mbmon_check_func)();
 #include "sysdeps/gtop.c"
 #endif
 
+#if defined(WIN32)
+#include "sysdeps/win32.c"
+#endif
+
+#if !defined(WIN32)
 #include <sys/utsname.h>
+#endif
+
+#if !defined(SENSORS_COMMON) && !defined(WIN32)
+static gboolean (*mbmon_check_func)();
+#endif
 
 gchar *
 gkrellm_sys_get_host_name(void)
@@ -77,6 +104,7 @@ gkrellm_sys_get_host_name(void)
 	return buf;
 	}
 
+#if !defined(WIN32)
 gchar *
 gkrellm_sys_get_system_name(void)
 	{
@@ -89,12 +117,13 @@ gkrellm_sys_get_system_name(void)
 		sname = g_strdup("unknown name");
 	return sname;
 	}
+#endif
 
 gboolean
 gkrellm_sys_sensors_mbmon_port_change(gint port)
 	{
 	gboolean	result = FALSE;
-
+#if !defined(WIN32)
 	_GK.mbmon_port = port;
 
 	/* mbmon_check_func will be set if sysdep code has included
@@ -107,11 +136,16 @@ gkrellm_sys_sensors_mbmon_port_change(gint port)
 		gkrellm_sensors_model_update();
 		gkrellm_sensors_rebuild(TRUE, TRUE, TRUE);
 		}
+#endif
 	return result;
 	}
 
 gboolean
 gkrellm_sys_sensors_mbmon_supported(void)
 	{
+#if !defined(WIN32)
 	return mbmon_check_func ? TRUE : FALSE;
+#else
+	return FALSE;
+#endif
 	}
