@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2008 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,6 +17,18 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -239,12 +251,15 @@ gkrellm_make_home_subdir(gchar *subdir, gchar **path)
 gchar *
 gkrellm_gtk_entry_get_text(GtkWidget **entry)
 	{
-	gchar	*s = "";
+	static /*const*/ gchar *def_s = "";
+	gchar *s = def_s;
 
 	if (*entry)
-		s = (gchar *) gtk_entry_get_text(GTK_ENTRY(*entry));
-	while (*s == ' ' || *s == '\t')
-		++s;
+		{
+		s = (gchar *)gtk_entry_get_text(GTK_ENTRY(*entry));
+		while (*s == ' ' || *s == '\t')
+			++s;
+		}
 	return s;
 	}
 
@@ -291,7 +306,7 @@ gkrellm_gdk_text_width(PangoFontDescription *font_desc,
 	pango_layout_set_text(layout, string, len);
 	pango_layout_get_pixel_size(layout, &w, &h);
 	g_object_unref(layout);
-	return w;	
+	return w;
 	}
 
 gint
@@ -306,7 +321,7 @@ gkrellm_gdk_text_markup_width(PangoFontDescription *font_desc,
 	pango_layout_set_markup(layout, string, len);
 	pango_layout_get_pixel_size(layout, &w, &h);
 	g_object_unref(layout);
-	return w;	
+	return w;
 	}
 
 gint
