@@ -4,31 +4,29 @@
 |  Author:  Bill Nalen    bill@nalens.com
 |  Latest versions might be found at:  http://bill.nalens.com
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
 
-
-// don't forget to populate this structure in win32.c
-// and redirect actual call in win32-plugin.c
-// changes will require recompile of gkrellm and all modules
-
 #ifndef WIN32_PLUGIN_H
 #define WIN32_PLUGIN_H
 
-typedef struct {
+#include "gkrellm.h"
+
+typedef struct
+{
       /* Data structure allocation
       */
     GkrellmChart *(*gkrellm_chart_new0)(void);
@@ -84,7 +82,7 @@ typedef struct {
 					    GdkPixmap *, gchar *);
     GkrellmChartdata *(*gkrellm_add_default_chartdata)(GkrellmChart *, gchar *);
     void		(*gkrellm_alloc_chartdata)(GkrellmChart *);
-    void		(*gkrellm_store_chartdata)(GkrellmChart *, gulong, gulong, gulong);
+    void		(*gkrellm_store_chartdata)(GkrellmChart *, gulong, ...);
     void		(*gkrellm_draw_chartdata)(GkrellmChart *);
     void		(*gkrellm_monotonic_chartdata)(GkrellmChartdata *, gboolean);
     gboolean	(*gkrellm_get_chartdata_hide)(GkrellmChartdata *);
@@ -139,8 +137,7 @@ typedef struct {
     void		(*gkrellm_panel_destroy)(GkrellmPanel *);
     void		(*gkrellm_panel_hide)(GkrellmPanel *);
     void		(*gkrellm_panel_show)(GkrellmPanel *);
-    gboolean	(*gkrellm_panel_enable_visibility)(GkrellmPanel *p, gboolean,
-						    gboolean *);
+    gboolean	(*gkrellm_panel_enable_visibility)(GkrellmPanel *p, gboolean, gboolean *);
     gboolean	(*gkrellm_is_panel_visible)(GkrellmPanel *);
     void		(*gkrellm_panel_keep_lists)(GkrellmPanel *, gboolean);
     void		(*gkrellm_draw_panel_label)(GkrellmPanel *);
@@ -366,7 +363,7 @@ typedef struct {
       */
     GdkGC		*(*gkrellm_draw_GC)(gint);
     GdkGC		*(*gkrellm_bit_GC)(gint);
-    GdkFont		*(*gkrellm_default_font)(gint);
+    PangoFontDescription		*(*gkrellm_default_font)(gint);
     GdkColor	*(*gkrellm_white_color)(void);
     GdkColor	*(*gkrellm_black_color)(void);
     GdkColor	*(*gkrellm_in_color)(void);
@@ -520,8 +517,77 @@ typedef struct {
         				gint header_pad, gint box_pad, gboolean pack_start);
 	void		(*gkrellm_remove_launcher)(GkrellmLauncher *launch);
 
+
+  //---------------------------------------------------------------------------
+  // new since 2.2.0
+
+  void    (*gkrellm_decal_get_size)(GkrellmDecal *d, gint *w, gint *h);
+  void    (*gkrellm_decal_text_set_offset)(GkrellmDecal *d, gint x, gint y);
+  void    (*gkrellm_decal_text_get_offset)(GkrellmDecal *d, gint *x, gint *y);
+  void    (*gkrellm_chart_reuse_text_format)(GkrellmChart *cp);
+  gchar  *(*gkrellm_get_hostname)(void);
+
+  void    (*gkrellm_decal_scroll_text_set_text)(GkrellmPanel *p, GkrellmDecal *d, gchar *text);
+  void    (*gkrellm_decal_scroll_text_get_size)(GkrellmDecal *d, gint *w, gint *h);
+  void    (*gkrellm_decal_scroll_text_align_center)(GkrellmDecal *d, gboolean center);
+  void    (*gkrellm_decal_scroll_text_horizontal_loop)(GkrellmDecal *d, gboolean loop);
+  void    (*gkrellm_decal_scroll_text_vertical_loop)(GkrellmDecal *d, gboolean loop);
+
+  void    (*gkrellm_text_extents)(PangoFontDescription *font_desc, gchar *text, gint len, gint *width, gint *height, gint *baseline, gint *y_ink);
+
+  gint    (*gkrellm_gdk_string_width)(PangoFontDescription *, gchar *);
+  void    (*gkrellm_gdk_draw_string)(GdkDrawable *drawable, PangoFontDescription *font, GdkGC *gc, gint x, gint y, gchar *string);
+  void    (*gkrellm_gdk_draw_text)(GdkDrawable *drawable, PangoFontDescription *font, GdkGC *gc, gint x, gint y, gchar *string, gint len);
+
+  gboolean  (*gkrellm_client_mode)(void);
+  void      (*gkrellm_client_plugin_get_setup)(gchar *key_name, void (*setup_func_cb)(gchar *str));
+  void      (*gkrellm_client_plugin_serve_data_connect)(GkrellmMonitor *mon, gchar *key_name, void (*func_cb)(gchar *line));
+  //void      (*gkrellm_client_plugin_reconnect_connect)(gchar *key_name, void (*func_cb)()); // FIXME: missing in gkrellm?
+
+
+  //---------------------------------------------------------------------------
+  // new since 2.2.1
+
+  void    (*gkrellm_draw_decal_markup)(GkrellmPanel *p, GkrellmDecal *d, gchar *text);
+  void    (*gkrellm_decal_scroll_text_set_markup)(GkrellmPanel *p, GkrellmDecal *d, gchar *text);
+
+
+  //---------------------------------------------------------------------------
+  // new since 2.2.2
+
+  void    (*gkrellm_panel_label_get_position)(GkrellmStyle *style, gint *x_position, gint *y_off);
+
+
+  //---------------------------------------------------------------------------
+  // new since 2.2.5
+
+  gboolean      (*gkrellm_client_send_to_server)(gchar *key_name, gchar *line);
+  
+  GkrellmDecal *(*gkrellm_create_decal_text_markup)(GkrellmPanel *p, gchar *string, GkrellmTextstyle *ts, GkrellmStyle *style, gint x, gint y, gint w);
+  void          (*gkrellm_decal_text_markup_insert)(GkrellmDecal *d, gchar *s, GkrellmTextstyle *ts, gint x_off, gint y_off);
+  
+  void          (*gkrellm_decal_text_nth_inserted_set_offset)(GkrellmDecal *d, gint n, gint x_off, gint y_off);
+  void          (*gkrellm_decal_text_nth_inserted_get_offset)(GkrellmDecal *d, gint n, gint *x_off, gint *y_off);
+  void          (*gkrellm_config_instant_apply)(GkrellmMonitor *mon);
+  GtkTreeSelection *(*gkrellm_gtk_scrolled_selection)(GtkTreeView *treeview, GtkWidget *box, GtkSelectionMode s_mode, GtkPolicyType h_policy, GtkPolicyType v_policy, void (*func_cb)(), gpointer data);
+  void          (*gkrellm_text_markup_extents)(PangoFontDescription *font_desc, gchar *text, gint len, gint *width, gint *height, gint *baseline, gint *y_ink);
+  gint          (*gkrellm_gdk_string_markup_width)(PangoFontDescription *, gchar *);
+  gint          (*gkrellm_gdk_text_markup_width)(PangoFontDescription *font_desc, const gchar *string, gint len);
+  void          (*gkrellm_gdk_draw_string_markup)(GdkDrawable *drawable, PangoFontDescription *font, GdkGC *gc, gint x, gint y, gchar *string);
+  void          (*gkrellm_gdk_draw_text_markup)(GdkDrawable *drawable, PangoFontDescription *font, GdkGC *gc, gint x, gint y, gchar *string, gint len);
+
+
 } win32_plugin_callbacks;
 
-extern win32_plugin_callbacks* callbacks;
 
-#endif
+/// part of win32-plugin.c
+/// 
+extern win32_plugin_callbacks gkrellm_callbacks;
+
+
+/// \brief initializes \p gkrellm_callbacks
+/// Has to be called at gkrellm startup before loading plugins.
+/// \note only needed on win32
+void win32_init_callbacks(void);
+
+#endif // WIN32_PLUGIN_H
